<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Professional Subscription
 * Agreement ("License") which can be viewed at
 * http://www.sugarcrm.com/crm/products/sugar-professional-eula.html
 * By installing or using this file, You have unconditionally agreed to the
 * terms and conditions of the License, and You may not use this file except in
 * compliance with the License.  Under the terms of the license, You shall not,
 * among other things: 1) sublicense, resell, rent, lease, redistribute, assign
 * or otherwise transfer Your rights to the Software, and 2) use the Software
 * for timesharing or service bureau purposes such as hosting the Software for
 * commercial gain and/or for the benefit of a third party.  Use of the Software
 * may be subject to applicable fees and any use of the Software without first
 * paying applicable fees is strictly prohibited.  You do not have the right to
 * remove SugarCRM copyrights from the source code or user interface.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *  (i) the "Powered by SugarCRM" logo and
 *  (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * Your Warranty, Limitations of liability and Indemnity are expressly stated
 * in the License.  Please refer to the License for the specific language
 * governing these rights and limitations under the License.  Portions created
 * by SugarCRM are Copyright (C) 2004-2007 SugarCRM, Inc.; All Rights Reserved.
 ********************************************************************************/

require_once('include/Sugar_Smarty.php');

class DashletsDialog {
	var $dashlets = array();
	
    function getDashlets($category='') {
        global $app_strings, $current_language, $mod_strings;
        
        require_once('cache/dashlets/dashlets.php');
        
        $categories = array( 'module' 	=> 'Module Views',
        					 'portal' 	=> 'Portal',
        					 'charts'	=> 'Charts',
        					 'tools'	=> 'Tools',
        					 'misc'		=> 'Miscellaneous' );
        
        $dashletStrings = array();
        $dashletsList = array();
        
        if (!empty($category)){
			$dashletsList[$categories[$category]] = array();        	
        }
        else{
	        $dashletsList['Module Views'] = array();
	        $dashletsList['Charts'] = array();
	        $dashletsList['Tools'] = array();
        }
              
        asort($dashletsFiles);
        foreach($dashletsFiles as $className => $files) {
            if(!empty($files['meta']) && is_file($files['meta'])) {
                require_once($files['meta']); // get meta file
                
                $directory = substr($files['meta'], 0, strrpos($files['meta'], '/') + 1);
                if(is_file($directory . $files['class'] . '.' . $current_language . '.lang.php')) 
                    require_once($directory . $files['class'] . '.' . $current_language . '.lang.php');
                elseif(is_file($directory . $files['class'] . '.en_us.lang.php')) 
                    require_once($directory . $files['class'] . '.en_us.lang.php');

                // try to translate the string
                if(empty($dashletStrings[$files['class']][$dashletMeta[$files['class']]['title']]))
                    $title = $dashletMeta[$files['class']]['title'];
                else 
                    $title = $dashletStrings[$files['class']][$dashletMeta[$files['class']]['title']];
                
                // try to translate the string
                if(empty($dashletStrings[$files['class']][$dashletMeta[$files['class']]['description']]))
                    $description = $dashletMeta[$files['class']]['description']; 
                else 
                    $description = $dashletStrings[$files['class']][$dashletMeta[$files['class']]['description']];

				// generate icon
                if (!empty($dashletMeta[$files['class']]['icon'])){
                	$icon = '<img src="' . $dashletMeta[$files['class']]['icon'] .'" border="0" align="absmiddle" />';
                }
                else{
	                if (empty($dashletMeta[$files['class']]['module'])){
                		$icon = get_dashlets_dialog_icon('default');
                	}
					else{
						if((!in_array($dashletMeta[$files['class']]['module'], $GLOBALS['moduleList']) && !in_array($dashletMeta[$files['class']]['module'], $GLOBALS['modInvisList'])) && (!in_array('Activities', $GLOBALS['moduleList']) || !in_array($dashletMeta[$files['class']]['module'], $GLOBALS['modInvisListActivities']))){
							unset($dashletMeta[$files['class']]);
							continue;
						}else{
	                    	$icon = get_dashlets_dialog_icon($dashletMeta[$files['class']]['module']);
						}
                	}
                }
                
                // determine whether to display
                if (!empty($dashletMeta[$files['class']]['hidden']) && $dashletMeta[$files['class']]['hidden'] === true){
                	$displayDashlet = false;
                }
                else{
                	$displayDashlet = true;
                }
                
                if ($dashletMeta[$files['class']]['category'] == 'Charts'){
                	$type = 'predefined_chart';
                }
                else{
                	$type = 'module';
                }
                	
                if ($displayDashlet){    
					$cell = array( 'title' => $title,
								   'description' => $description,
								   'onclick' => 'return SUGAR.mySugar.addDashlet(\'' . $className . '\', \'' . $type . '\');',
								   'icon' => $icon, );

	                if (!empty($category) && $dashletMeta[$files['class']]['category'] == $categories[$category]){
	                	array_push($dashletsList[$categories[$category]], $cell);
	                }
	                else if (empty($category)){
						array_push($dashletsList[$dashletMeta[$files['class']]['category']], $cell);
	                }
                }
            }
        }
        if (!empty($category)){
        	asort($dashletsList[$categories[$category]]);
        }
        else{
        	foreach($dashletsList as $key=>$value){
        		asort($dashletsList[$key]);
        	}
        }
        $this->dashlets = $dashletsList;        
    }
        

    function getReportCharts($category){
    	global $current_user;
    	
    	require_once('modules/Reports/SavedReport.php');
    	//require_once('modules/Reports/Report.php');
    	
    	$chartsList = array();
    	$focus = new SavedReport();
    	$focus->disable_row_level_security = false;
    	
    	switch($category){
    		case 'global':
		    	// build global where string
		    	$where = "AND saved_reports.team_id='1'";
    	    	break;
    	    	
    		case 'myTeams':
		    	// build myTeams where string
		    	$myTeams = $current_user->get_my_teams();
		    	$where = '';
		    	foreach($myTeams as $team_id=>$team_name){
		    		if ($team_id != '1' && $team_id != $current_user->getPrivateTeamID()){
			    		if ($where == ''){
			    			$where .= 'AND ';
			    		}
			    		else{
			    			$where .= 'OR ';
			    		}
		    			$where .= "saved_reports.team_id='".$team_id. "' ";
		    		}
		    	}
		    	break;
		    	
    		case 'mySaved':
		    	// build mySaved where string
		    	$where = "AND saved_reports.team_id='".$current_user->getPrivateTeamID()."'";
		    	break;
		    	
		    case 'myFavorites':
		    	global $current_user;
		    	$current_favorites = $current_user->getPreference('favorites', 'Reports');
				$current_favorite_ids = !empty($current_favorites) ? $current_favorites : array();
		    	$where = 'AND saved_reports.id IN (\'' . implode("','", array_keys($current_favorite_ids)) . '\')';
		    	break;
		    	
    		default:
    			break;
    	}
		
    	$savedReports = $focus->get_full_list(""," chart_type != 'none' " . $where);
		
		$chartsList = array();
		if (!empty($savedReports)){
			foreach($savedReports as $savedReport){
				$report_def = array( 'title' => $savedReport->name,
									 'onclick' => 'return SUGAR.mySugar.addDashlet(\'' . $savedReport->id . '\', \'chart\');',
									);
									 
				array_push($chartsList, $report_def);
			}
		}
		asort($chartsList);
		$this->dashlets[$category] = $chartsList;
    }

}

?>
