<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Professional Subscription
 * Agreement ("License") which can be viewed at
 * http://www.sugarcrm.com/crm/products/sugar-professional-eula.html
 * By installing or using this file, You have unconditionally agreed to the
 * terms and conditions of the License, and You may not use this file except in
 * compliance with the License.  Under the terms of the license, You shall not,
 * among other things: 1) sublicense, resell, rent, lease, redistribute, assign
 * or otherwise transfer Your rights to the Software, and 2) use the Software
 * for timesharing or service bureau purposes such as hosting the Software for
 * commercial gain and/or for the benefit of a third party.  Use of the Software
 * may be subject to applicable fees and any use of the Software without first
 * paying applicable fees is strictly prohibited.  You do not have the right to
 * remove SugarCRM copyrights from the source code or user interface.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *  (i) the "Powered by SugarCRM" logo and
 *  (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * Your Warranty, Limitations of liability and Indemnity are expressly stated
 * in the License.  Please refer to the License for the specific language
 * governing these rights and limitations under the License.  Portions created
 * by SugarCRM are Copyright (C) 2004-2007 SugarCRM, Inc.; All Rights Reserved.
 ********************************************************************************/

/**
 * DetailViewMetaParser.php
 * This is a utility file that attempts to provide support for parsing pre 5.0 SugarCRM 
 * DetailView.html files and produce a best guess detailviewdefs.php file equivalent.
 * 
 * @author Collin Lee
 */
require_once('include/SugarFields/Parsers/MetaParser.php');

/**
 * DetailViewMetaParser.php
 * This class is responsible for handling the parsing of DetailView.html files from
 * SugarCRM versions prior to 5.x.  It will make a best guess of translating the
 * HTML file to a metadata format.
 * 
 * @author Collin Lee
 */
class DetailViewMetaParser extends MetaParser {

function DetailViewMetaParser() {
   $this->mView = 'DetailView';
}


/**
 * parse
 * 
 * @param $filePath The file path of the HTML file to parse
 * @param $vardefs The module's vardefs
 * @param $moduleDir The module's directory
 * @param $merge boolean value indicating whether or not to merge the parsed contents
 * @param $masterCopy The file path of the mater copy of the metadata file to merge against
 * @return String format of metadata contents
 **/
function parse($filePath, $vardefs = array(), $moduleDir = '', $merge=false, $masterCopy=null) {
   
// Grab file contents
$contents = file_get_contents($filePath);
   
// Remove \n,\r characters to allow for better text parsing
$contents = $this->trimHTML($contents);
   
// Get all the tables
$tables = $this->getElementsByType("table", $contents);
   
// Skip the first one
$tables = array_slice($tables, 1);

$panels = array();
$tableCount = 0;
$metarow = array();

foreach($tables as $table) {   
               
   $tablerows = $this->getElementsByType("tr", $table);

   foreach($tablerows as $trow) {

       $metacolumns = array();  
   	   $columns = $this->getElementsByType("td", $trow);

   	   // We reverse the array since we want to first process the slot[0-9]+b tags
   	   $tablecolumns = array_reverse($columns, true);
   	   
       $def = null;
       $slot = 0;
	   foreach($tablecolumns as $tcols) {

	   	  $field = strtolower($this->getElementValue("span", $tcols));
	   	  if(empty($field) || $field == '') {
	   	  	 $field = $tcols;
	   	  }
	   	  //If there are HTML tags, do not use as field, but only get contents between {} tags
	   	  if(preg_match('/^[<][^>]*?[>]([\{][^\}].*?[\}])<\/.*?/s', $field, $m)) { 
	   	  	 $field = $m[1];
	   	  } //if

	   	  $sugarAttrValue = $this->getTagAttribute("sugar", $tcols, "'slot[0-9]+b$'");
	      $sugarAttrValue = is_array($sugarAttrValue) && count($sugarAttrValue) != 0 ? $sugarAttrValue : ($slot % 2 == 0) ? true : false;
	      $slot++;

          //Grab all values enclosed within {} tags
          preg_match_all('/[\{]([^\}].*?)[\}]/s', $field, $matches, PREG_SET_ORDER);
	   	  
	   	  if($sugarAttrValue) {         
	   	  	 if($matches) {	 
	   	  	 	 //Check if there are more than one tag with {} enclosed, treat as custom code
		   	  	 if(count($matches) > 1) {
		   	  	 	 // Keep $mod_strings and $app_strings as uppercased variables
		   	  	 	 
		   	  	 	 foreach($matches as $tag[1]) {
		   	  	 	    if(preg_match("/^(mod[\.]|app[\.]).*?/s", $tag[1][1])) {
		   	  	 	       $field = str_replace($tag[1][1], '$'.strtoupper($tag[1][1]), $field);      
		   	  	 	    } else {
		   	  	 	       if(!empty($vardefs[$tag[1][1]])) {
		   	  	 	          $field = str_replace($tag[1][1], '$fields.'.$tag[1][1].'.value', $field);
		   	  	 	       } else {
		   	  	 	       	  $phpName = $this->findAssignedVariableName($tag[1][1], $filePath);
		   	  	 	       	  $field = str_replace($tag[1][1], '$fields.'.$phpName.'.value', $field);
		   	  	 	       } //if-else
		   	  	 	    }
		   	  	 	 }
		   	  	 	 
		   	  	 	 $def = array();
		   	  	 	 $def['name'] = isset($phpName) ? $phpName : $tag[1][1];
		   	  	 	 $def['customCode'] = $field; 
		   	  	 } else {

			   	  	 foreach($matches as $tag[1]) {
			   	  	 	     if(!empty($tag[1][0])) {
			   	  	 	        if(!empty($vardefs[$tag[1][1]])) {
			   	  	 	        	$name = $tag[1][1];
			   	  	 	        	//Check if field type is text
			   	  	 	        	if(!empty($vardefs[$name]['type']) && $vardefs[$name]['type'] == 'text') {
			   	  	 	        	  $def = array();
			   	  	 	        	  $def['name'] = $name;	    	
			   	  	 	        	} else {
			   	  	 	        	  $def = $name;
			   	  	 	        	}
			   	  	 	        } else {
			   	  	 	        	//At this point we don't have a tag that matches so let's try to 
			   	  	 	        	//see if the PHP file attempted a direct variable assignment from $focus instance
			   	  	 	        	$def = $this->findAssignedVariableName($tag[1][1], $filePath);
			   	  	 	            //$def = $tag[1][1];
			   	  	 	        } //if-else
			   	  	 	    } //if
			   	  	 } //foreach

		   	  	 } //if-else
		   	  	 $metacolumns[] = $def;
	   	  	 }
	   	  } else {
	   	  	 if(isset($def)) {
	   	  	 	preg_match_all('/[\{](MOD[\.]|APP[\.])LBL_([^\}].*?)[\}]/si', $field, $matches, PREG_SET_ORDER);
	   	  	 	if($matches && count($matches[0]) == 3) {
		   	  	 	$label = $matches[0][2];

                    $reset = false;
		   	  	 	if(is_array($def) && !isset($def['label']) && $def['name'] != $label) {
		   	  	 	   $def2 = $def;
		   	  	 	   $def2['label'] = 'LBL_' . strtoupper($label);
		   	  	 	   $reset = true;	
		   	  	 	} else if(!is_array($def) && $def != $label) {
		   	  	 	   $temp = $def;
		   	  	 	   $def2 = array();
		   	  	 	   $def2['name'] = $temp;
		   	  	 	   $def2['label'] = 'LBL_' . strtoupper($label);
		   	  	 	   $reset = true;
		   	  	 	} //if-else
		   	  	 	
		   	  	 	// Reset the last entry in the Array to use the new label attributes
		   	  	 	if($reset) {
		   	  	 	   array_pop($metacolumns);
		   	  	 	   $metacolumns[] = $def2;
		   	  	 	} //if
	   	  	 	} //if
	   	  	 }
	   	  } //if-else
	   	  
	   } //foreach($tablecolumns as $tcols)

   	   $metarow[] = array_reverse($metacolumns);
   } //foreach($tablerows as $trow) 
   
   
   $id = $tableCount == 0 ? 'default' : $tableCount;
   $tableCount++;
   $panels[$id] = $metarow;
   
} //foreach($tables as $table)

$templateMeta = array();
   
if($merge && !empty($masterCopy) && file_exists($masterCopy)) {

   $panels = $this->mergePanels($panels, $moduleDir, $masterCopy);
   $templateMeta = $this->mergeTemplateMeta($templateMeta, $moduleDir, $masterCopy);
}

return $this->createFileContents($moduleDir, $panels, $templateMeta, $filePath);

}

}
?>

