<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Professional Subscription
 * Agreement ("License") which can be viewed at
 * http://www.sugarcrm.com/crm/products/sugar-professional-eula.html
 * By installing or using this file, You have unconditionally agreed to the
 * terms and conditions of the License, and You may not use this file except in
 * compliance with the License.  Under the terms of the license, You shall not,
 * among other things: 1) sublicense, resell, rent, lease, redistribute, assign
 * or otherwise transfer Your rights to the Software, and 2) use the Software
 * for timesharing or service bureau purposes such as hosting the Software for
 * commercial gain and/or for the benefit of a third party.  Use of the Software
 * may be subject to applicable fees and any use of the Software without first
 * paying applicable fees is strictly prohibited.  You do not have the right to
 * remove SugarCRM copyrights from the source code or user interface.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *  (i) the "Powered by SugarCRM" logo and
 *  (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * Your Warranty, Limitations of liability and Indemnity are expressly stated
 * in the License.  Please refer to the License for the specific language
 * governing these rights and limitations under the License.  Portions created
 * by SugarCRM are Copyright (C) 2004-2007 SugarCRM, Inc.; All Rights Reserved.
 ********************************************************************************/

/**
 * EdtiViewMetaParser.php
 * This is a utility file that attempts to provide support for parsing pre 5.0 SugarCRM 
 * EditView.html files and produce a best guess editviewdefs.php file equivalent.
 * 
 * @author Collin Lee
 */
  
require_once('include/SugarFields/Parsers/MetaParser.php');
require_once('include/language/en_us.lang.php');

class EditViewMetaParser extends MetaParser {

function EditViewMetaParser() {
   $this->mView = 'EditView';
}

/**
 * parse
 * 
 * @param $filePath The file path of the HTML file to parse
 * @param $vardefs The module's vardefs
 * @param $moduleDir The module's directory
 * @param $merge boolean value indicating whether or not to merge the parsed contents
 * @param $masterCopy The file path of the mater copy of the metadata file to merge against
 * @return String format of metadata contents
 **/
function parse($filePath, $vardefs = array(), $moduleDir = '', $merge=false, $masterCopy=null) {

   global $app_strings;
   $contents = file_get_contents($filePath);
   $contents = $this->trimHTML($contents);
   $moduleName = '';
   
   $tables = $this->getElementsByType("table", $contents);
   $formElements = $this->getFormElements($tables[0]);
   $hiddenInputs = array();
   foreach($formElements as $elem) {
   	      $type = $this->getTagAttribute("type", $elem);
   	      if(preg_match('/hidden/si',$type)) {
   	         $name = $this->getTagAttribute("name", $elem);
   	         $value = $this->getTagAttribute("value", $elem);
   	         $hiddenInputs[$name] = $value;
   	      }
   }

   // Get the second table in the page and onward
   $tables = array_slice($tables, 1);
   $panels = array();
   $tableCount = 0;
   $addedElements = array();
   
   foreach($tables as $table) {
   	      $toptr = $this->getElementsByType("tr", $table);         
          foreach($toptr as $tr) {
          	      $tabledata = $this->getElementsByType("table", $tr);
          	      $data = array();
          	      $panelKey = $tableCount == 0 ? "default" : $tableCount;
          	      foreach($tabledata as $t) {
          	      	      $vals = array_values($this->getElementsByType("tr", $t));
          	      	      if(preg_match_all('/<h4[^>]*?>.*?{(MOD|APP)[\.](.*?)}<\/h4>/si', $vals[0], $matches, PREG_SET_ORDER)) {
          	      	      	array_shift($vals);
          	      	      	$panelKey = count($matches[0]) == 3 ? strtolower($matches[0][2]) : $panelKey;
          	      	      } 
          	      	      $panels[$panelKey] = $vals; 
   	              } //foreach    
          } //foreach
          $tableCount++;
   } //foreach

   foreach($panels as $id=>$tablerows) {
   	
       $metarow = array();
       	       
	   foreach($tablerows as $trow) {
	   	
	   	   $emptyCount = 0;
	   	   $tablecolumns = $this->getElementsByType("td", $trow);
	       $col = array();
	       $slot = 0;
	       
		   foreach($tablecolumns as $tcols) {
		   	  	 	  
		   	  $sugarAttrLabel = $this->getTagAttribute("sugar", $tcols, "'^slot[^b]+$'");
		   	  $sugarAttrValue = $this->getTagAttribute("sugar", $tcols, "'slot[0-9]+b$'");

              // If there wasn't any slot numbering/lettering then just default to expect label->vallue pairs
	          $sugarAttrLabel = count($sugarAttrLabel) != 0 ? $sugarAttrLabel : ($slot % 2 == 0) ? true : false;
	          $sugarAttrValue = count($sugarAttrValue) != 0 ? $sugarAttrValue : ($slot % 2 == 1) ? true : false;
	          $slot++;
	          
              if($sugarAttrValue) {
		   	     	 
		   	  	 $spanValue = strtolower($this->getElementValue("span", $tcols));
                 if(empty($spanValue)) {
                    $spanValue = strtolower($this->getElementValue("slot", $tcols));	
                 }
                 if(empty($spanValue)) {
                    $spanValue = strtolower($this->getElementValue("td", $tcols));
                 }
                 

                 //Get all the editable form elements' names
		   	  	 $formElementNames = $this->getFormElementsNames($spanValue);		   	  	 
		   	  	 $customField = $this->getCustomField($formElementNames);
		   	  	 
		   	  	 $name = '';
                 $readOnly = false;
                 $fields = null;
                 $customCode = null;
                            
                 if(!empty($customField)) {           
                   // If it's a custom field we just set the name
                   $name = $customField;
                         
                 } else if(empty($formElementNames) && preg_match_all('/[\{]([^\}]*?)[\}]/s', $spanValue, $matches, PREG_SET_ORDER)) {
		   	  	   // We are here if the $spanValue did not contain a form element for editing.
		   	  	   // We will assume that it is read only (since there were no edit form elements)
		   	  	   

		           // If there is more than one matching {} value then try to find the right one to key off
		           // based on vardefs.php file.  Also, use the entire spanValue as customCode
		           	if(count($matches) > 1) {
				       $name = $matches[0][1];  
				       $customCode = $spanValue;
				       foreach($matches as $pair) {
			   	  	 	   if(preg_match("/^(mod[\.]|app[\.]).*?/s", $pair[1])) {
			   	  	 	       $customCode = str_replace($pair[1], '$'.strtoupper($pair[1]), $customCode);      
			   	  	 	   } else {
			   	  	 	       if(!empty($vardefs[$pair[1]])) {
			   	  	 	       	  $name = $pair[1];
			   	  	 	          $customCode = str_replace($pair[1], '$fields.'.$pair[1].'.value', $customCode);
			   	  	 	       } else {
			   	  	 	       	  $phpName = $this->findAssignedVariableName($pair[1], $filePath);
			   	  	 	       	  $customCode = str_replace($pair[1], '$fields.'.$phpName.'.value', $customCode);
			   	  	 	       } //if-else
			   	  	 	   }
			           } //foreach
			       } else {
			       	   //If it is only a label, skip
			       	   if(preg_match("/^(mod[\.]|app[\.]).*?/s", $matches[0][1])) {
			       	   	  continue;
			       	   }
			   	  	   $name = $matches[0][1];    
			   	   }
			   	  	   
			   	   $readOnly = true;  
		   	  	 } else if(is_array($formElementNames)) {
                      
		   	  	      if(count($formElementNames) == 1) {

		   	  	      	 if(!empty($vardefs[$formElementNames[0]])) {
		   	  	            $name = $formElementNames[0];
		   	  	      	 } else {
		   	  	      	 	// Try to use the EdtiView.php file to find author's intent
		   	  	      	 	$name = $this->findAssignedVariableName($formElementNames[0], $filePath);

		   	  	      	 	//If it's still empty, just use the entire block as customCode
		   	  	      	 	if(empty($vardefs[$name])) {
		   	  	      	 	   //Replace any { characters just in case   
		   	  	      	 	   $customCode = str_replace('{', '{$', $spanValue);
		   	  	      	 	}
		   	  	      	 } //if-else
		   	  	      } else {
		   	  	      	 $fields = array();
		   	  	      	 foreach($formElementNames as $elementName) {
		   	  	      	 	// What we are doing here is saying that we will add all your fields assuming
		   	  	      	 	// there are none that are of type relate or link.  However, if we find such a type
		   	  	      	 	// we'll take the first one found and assume that is the field you want (the SugarFields
		   	  	      	 	// library will handle rendering the popup and select and clear buttons for you).
		   	  	      	 	if(isset($vardefs[$elementName])) {
		   	  	      	 	   $type = $vardefs[$elementName]['type'];
		   	  	      	 	   if($type != 'relate' && $type != 'link') {
		   	  	      	 	      $fields[] = $elementName;
		   	  	      	 	      $name = $elementName;
		   	  	      	 	   } else {
		   	  	      	          unset($fields);
		   	  	      	 	      $name = $elementName;
		   	  	      	 	      break;	
		   	  	      	 	   }
		   	  	      	 	} else {
		   	  	      	 	   $name = $this->findAssignedVariableName($name, $filePath);
		   	  	      	 	   $fields[] = $name;
		   	  	      	 	}
		   	  	      	 }
		   	  	      } //if-else
		   	  	 }
		   	  	 
		   	  	 // Build the entry
		   	  	 if(preg_match("/<textarea/si", $spanValue)) {
		   	  	 	//special case for textarea form elements (add the displayParams)
		   	  	 	$displayParams = array();
		   	  	    $displayParams['rows'] = $this->getTagAttribute("rows", $spanValue);
		   	  	    $displayParams['cols'] = $this->getTagAttribute("cols", $spanValue);

		   	  	    if(!empty($displayParams['rows']) && !empty($displayParams['cols'])) {
			   	  	    $field = array();
			   	  	    $field['name'] = $name;
						$field['displayParams'] = $displayParams;
		   	  	    } else {
		   	  	        $field = $name;	
		   	  	    }
		   	  	 } else if($readOnly) {
		   	  	 	$field = array();
		   	  	 	$field['name'] = $name;
		   	  	 	$field['type'] = 'readonly';
		   	  	 	if(isset($customCode)) {
		   	  	 	   $field['customCode'] = $customCode;
		   	  	 	} //if
		   	  	 } else {

		   	  	 	if(isset($fields) || isset($customCode)) {
		   	  	 	   $field = array();
		   	  	 	   $field['name'] = $name;
		   	  	 	   if(isset($fields)) {
		   	  	 	   	  $field['fields'] = $fields;
		   	  	 	   }
		   	  	 	   if(isset($customCode)) {
		   	  	 	   	  $field['customCode'] = $customCode;
		   	  	 	   }
		   	  	 	} else {
		   	  	 	   $emptyCount = $name == '' ? $emptyCount + 1 : $emptyCount;
		   	  	 	   $field = $name;
		   	  	 	}	
		   	  	 } //if-else if-else block
		   	  	 
		   	  	 $addedField = is_array($field) ? $field['name'] : $field;
		   	  	 if(empty($addedField) || empty($addedElements[$addedField])) {
		   	  	  	$col[] = $field;
		   	  	  	$addedElements[$addedField] = $addedField;
		   	  	 }
		   	  	 
		   	  } //if($sugarAttrValue)
		   } //foreach
		   
		   // One last final check.  If $emptyCount does not equal Array $col count, don't add 
		   if($emptyCount != count($col)) {
	   	      $metarow[] = $col;
		   } //if
	   } //foreach
	  
	   $panels[$id] = $metarow;
	  
   } //foreach

   $templateMeta = array();
   if($merge && !empty($masterCopy) && file_exists($masterCopy)) {
      $panels = $this->mergePanels($panels, $moduleDir, $masterCopy);
      $templateMeta = $this->mergeTemplateMeta($templateMeta, $moduleDir, $masterCopy);
   }

   return $this->createFileContents($moduleDir, $panels, $templateMeta, $filePath);
}


}
?>
