<?php
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Professional Subscription
 * Agreement ("License") which can be viewed at
 * http://www.sugarcrm.com/crm/products/sugar-professional-eula.html
 * By installing or using this file, You have unconditionally agreed to the
 * terms and conditions of the License, and You may not use this file except in
 * compliance with the License.  Under the terms of the license, You shall not,
 * among other things: 1) sublicense, resell, rent, lease, redistribute, assign
 * or otherwise transfer Your rights to the Software, and 2) use the Software
 * for timesharing or service bureau purposes such as hosting the Software for
 * commercial gain and/or for the benefit of a third party.  Use of the Software
 * may be subject to applicable fees and any use of the Software without first
 * paying applicable fees is strictly prohibited.  You do not have the right to
 * remove SugarCRM copyrights from the source code or user interface.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *  (i) the "Powered by SugarCRM" logo and
 *  (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * Your Warranty, Limitations of liability and Indemnity are expressly stated
 * in the License.  Please refer to the License for the specific language
 * governing these rights and limitations under the License.  Portions created
 * by SugarCRM are Copyright (C) 2004-2007 SugarCRM, Inc.; All Rights Reserved.
 * *******************************************************************************/
class SugarFieldHandler {
    var $sugarFieldObjects = array();
    
    function SugarFieldHandler() {
    }
    
    /**
     * return the singleton of the SugarField
     * 
     * @param field string field type
     */
    function getSugarField($field) {
        if(!isset($this->sugarFieldObjects[$field])) {
        	if(file_exists('include/SugarFields/Fields/' . $field . '/SugarField' . $field. '.php')){
           		$type = $field;
        	}else{
        		$type = 'Base';
        	}
        	
        	require_once('include/SugarFields/Fields/' . $type . '/SugarField' . $type . '.php');
            $class = 'SugarField' . $type; // ex. SugarFieldVarchar, SugarFieldInt, etc.
            $this->sugarFieldObjects[$field] = new $class($field); 
        }
        return $this->sugarFieldObjects[$field];
    }
        
    /**
     * Returns the smarty code to be used in a template built by TemplateHandler
     * The SugarField class is choosen dependant on the vardef's type field.
     * 
     * @param parentFieldArray string name of the variable in the parent template for the bean's data
     * @param vardef vardef field defintion
     * @param displayType string the display type for the field (eg DetailView, EditView, etc)
     * @param displayParam parameters for displayin
     *      available paramters are:
     *      * labelSpan - column span for the label
     *      * fieldSpan - column span for the field 
     */
    function displaySmarty($parentFieldArray, $vardef, $displayType, $displayParams = array(), $tabindex = 1) {
        $string = '';
        $displayType = 'get' . $displayType . 'Smarty'; // getDetailViewSmarty, getEditViewSmarty, etc...
		
		// This will handle custom type fields.
		// The incoming $vardef Array may have custom_type set.  
		// If so, set $vardef['type'] to the $vardef['custom_type'] value
		if(isset($vardef['custom_type'])) {
		   $vardef['type'] = $vardef['custom_type'];	
		}
		
        if(!empty($vardef['type'])) {
        	$type = $vardef['type'];
            switch($vardef['type']) {
               case 'float':
               case 'int':
                    $type = 'int';
                    break;
               case 'date':
                    $type = 'datetime';
                    break;
               case 'url':
               		$type = 'link';
               		break;   
            }
            $field = $this->getSugarField(ucfirst($type));
            $string = $field->$displayType($parentFieldArray, $vardef, $displayParams, $tabindex);

        }
        
        return $string;
    }
    
    
}

?>
