<?php
/**
 * The contents of this file are subject to the SugarCRM Professional Subscription
 * Agreement ("License") which can be viewed at
 * http://www.sugarcrm.com/crm/products/sugar-professional-eula.html
 * By installing or using this file, You have unconditionally agreed to the
 * terms and conditions of the License, and You may not use this file except in
 * compliance with the License.  Under the terms of the license, You shall not,
 * among other things: 1) sublicense, resell, rent, lease, redistribute, assign
 * or otherwise transfer Your rights to the Software, and 2) use the Software
 * for timesharing or service bureau purposes such as hosting the Software for
 * commercial gain and/or for the benefit of a third party.  Use of the Software
 * may be subject to applicable fees and any use of the Software without first
 * paying applicable fees is strictly prohibited.  You do not have the right to
 * remove SugarCRM copyrights from the source code or user interface.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *  (i) the "Powered by SugarCRM" logo and
 *  (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * Your Warranty, Limitations of liability and Indemnity are expressly stated
 * in the License.  Please refer to the License for the specific language
 * governing these rights and limitations under the License.  Portions created
 * by SugarCRM are Copyright (C) 2004-2007 SugarCRM, Inc.; All Rights Reserved.
 */

if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');



include_once('include/workflow/workflow_utils.php');
include_once('include/workflow/field_utils.php');
include_once('include/utils/expression_utils.php');

function process_workflow_actions(& $focus, $action_array){


	if($action_array['action_type']=="update"){
		process_action_update($focus, $action_array);
	}
	if($action_array['action_type']=="update_rel"){
		process_action_update_rel($focus, $action_array);
	}
	if($action_array['action_type']=="new"){
		process_action_new($focus, $action_array);
	}
	if($action_array['action_type']=="new_rel"){
		process_action_new_rel($focus, $action_array);
	}	
	
	
//end function process_workflow_actions
}

function process_action_update(&$focus, $action_array){
	
	foreach($action_array['basic'] as $field => $new_value){
		
		
		if(empty($action_array['basic_ext'][$field])){
			$focus->$field = convert_bool($new_value, $focus->field_defs[$field]['type']);
			$_POST[$field]=$focus->$field;
			execute_special_logic($field, $focus);
		}
		//otherwise rely on the basic_ext to handle the action for this field	
	}	

	foreach($action_array['basic_ext'] as $field => $new_value){
		
		//Only here if there is a datetime.
		if($new_value=='Triggered Date'){
			$focus->$field = get_expiry_date(get_field_type($focus->field_defs[$field]), $action_array['basic'][$field]);
			if($focus->field_defs[$field]['type']=='date' && !empty($focus->field_defs[$field]['rel_field']) ){
				$rel_field = $focus->field_defs[$field]['rel_field'];
				$focus->$rel_field = get_expiry_date('time', $action_array['basic'][$field]);	
			}
			execute_special_logic($field, $focus);
		}
		if($new_value=='Existing Value'){
			$focus->$field = get_expiry_date(get_field_type($focus->field_defs[$field]), $action_array['basic'][$field], true, $focus->$field);
			execute_special_logic($field, $focus);
		}	
	}	
	
	foreach($action_array['advanced'] as $field => $meta_array){
			
		$new_value = process_advanced_actions($focus, $field, $meta_array, $focus);
		$focus->$field = $new_value;	
		execute_special_logic($field, $focus);
	}	

//end function process_action_update
}	

function process_action_update_rel(& $focus, $action_array){

		$rel_handler = & $focus->call_relationship_handler("module_dir", true);
		$rel_handler->set_rel_vardef_fields($action_array['rel_module']);
		$rel_handler->build_info(false);
		
		$rel_list = $rel_handler->build_related_list();

	//All, First, Filter (default ALL)
	
	if(!empty($rel_list[0])){

		$rel_list = process_rel_type("rel_module_type", "rel_filter", $rel_list, $action_array);
		
		foreach($rel_list as $rel_object){
			
			foreach($action_array['basic'] as $field => $new_value){

				if(empty($action_array['basic_ext'][$field])){
					$rel_object->$field = convert_bool($new_value, $rel_object->field_defs[$field]['type']);
					$_POST[$field]=$rel_object->$field;
				}
				//otherwise rely on the basic_ext to handle the action for this field
				
			//loop through fields to change
			}
			foreach($action_array['basic_ext'] as $field => $new_value){
				//Only here if there is a datetime.
				if($new_value=='Triggered Date'){
					$rel_object->$field = get_expiry_date(get_field_type($rel_object->field_defs[$field]), $action_array['basic'][$field]);
				}
				if($new_value=='Existing Value'){
					$rel_object->$field = get_expiry_date(get_field_type($rel_object->field_defs[$field]), $action_array['basic'][$field], true, $rel_object->$field);
				}	
			}	
	
			foreach($action_array['advanced'] as $field => $meta_array){
				$new_value = process_advanced_actions($focus, $field, $meta_array, $rel_object);
				$rel_object->$field = $new_value;	
			}			
			$rel_object->save();	
		
		//end foreach rel_object	
		}		
		
	//end if there are any relationship records
	}	
		
//end function process_action_update_rel
}	

function process_action_new(& $focus, $action_array){
	
	//find out if the action_module is related to this module or not.  If so make sure to connect
	
	$target_module = get_module_info($action_array['action_module']);
	$rel_handler = $focus->call_relationship_handler("module_dir", true);
	//$rel_handler->base_bean = & $focus;
	$rel_handler->get_relationship_information($target_module);
	
	//get_relationship_information($target_module, $focus);

	foreach($action_array['basic'] as $field => $new_value){
			//rrs - bug 10466
			$target_module->$field = convert_bool($new_value, $target_module->field_defs[$field]['type'], (empty($target_module->field_defs[$field]['dbType']) ? '' : $target_module->field_defs[$field]['dbType']));
			$_POST[$field]=$target_module->$field;	
	//end foreach value
	}	
	foreach($action_array['basic_ext'] as $field => $new_value){
		//Only here if there is a datetime.
		if($new_value=='Triggered Date'){
			
			$target_module->$field = get_expiry_date(get_field_type($target_module->field_defs[$field]), $action_array['basic'][$field]);
		
			if($target_module->field_defs[$field]['type']=='date' && !empty($target_module->field_defs[$field]['rel_field']) ){
				$rel_field = $target_module->field_defs[$field]['rel_field'];
				$target_module->$rel_field = get_expiry_date('time', $action_array['basic'][$field]);	
			}
		}	
	}	
	
	foreach($action_array['advanced'] as $field => $meta_array){

			
		$new_value = process_advanced_actions($focus, $field, $meta_array, $target_module);
	
		$target_module->$field = $new_value;	
	}	
	clean_save_data($target_module, $action_array);
	
	
	//BEGIN BRIDGING FOR MEETINGS/CALLS
	if(!empty($action_array['bridge_id']) && $action_array['bridge_id']!=""){
		$target_module->bridge_id = $action_array['bridge_id'];
		$target_module->bridge_object = & $focus;
		
	}	
	//END BRIDGING FOR MEETINGS/CALLS

	$target_module->save();
	
//end function_action_new
}



function process_action_new_rel(& $focus, $action_array){

	///Build the relationship information using the Relationship handler
	$rel_handler = & $focus->call_relationship_handler("module_dir", true);
	$rel_handler->set_rel_vardef_fields($action_array['rel_module'], $action_array['action_module']);
	//$rel_handler->base_bean = & $focus;
	$rel_handler->build_info(true);
	//get related bean
	$rel_list = $rel_handler->build_related_list("base");

	
	//All, first, filter (FIRST)
	
	if(!empty($rel_list[0])){
		
		$rel_list = process_rel_type("rel_module_type", "rel_filter", $rel_list, $action_array);	
	
		foreach($rel_list as $rel_object){
			//Connect new module to the first related bean.
			$rel_handler2 = $rel_object->call_relationship_handler("module_dir", true);
			//$rel_handler->base_bean = & $rel_object;
			$rel_handler2->get_relationship_information($rel_handler->rel2_bean);			
				
			//get_relationship_information($rel_handler->rel2_bean, $rel_object);
			
			$target_module = & $rel_handler->rel2_bean;
			
			foreach($action_array['basic'] as $field => $new_value){
				$target_module->$field = convert_bool($new_value, $target_module->field_defs[$field]['type']);
				$_POST[$field]=$target_module->$field;	
			//end foreach value
			}	
			foreach($action_array['basic_ext'] as $field => $new_value){
				//Only here if there is a datetime.
				if($new_value=='Triggered Date'){
					
					$target_module->$field = get_expiry_date(get_field_type($target_module->field_defs[$field]), $action_array['basic'][$field]);
					if($target_module->field_defs[$field]['type']=='date' && !empty($target_module->field_defs[$field]['rel_field']) ){
						$rel_field = $target_module->field_defs[$field]['rel_field'];
						$target_module->$rel_field = get_expiry_date('time', $action_array['basic'][$field]);	
					}
				}	
			}	
	
			foreach($action_array['advanced'] as $field => $meta_array){	
				$new_value = process_advanced_actions($focus, $field, $meta_array, $target_module);
				$target_module->$field = $new_value;	
			}	
			clean_save_data($target_module, $action_array);

			
			//BEGIN BRIDGING FOR MEETINGS/CALLS
				if(!empty($action_array['bridge_id']) && $action_array['bridge_id']!=""){
					$target_module->bridge_id = $action_array['bridge_id'];
					$target_module->bridge_object = & $focus;
				}	
			//END BRIDGING FOR MEETINGS/CALLS	
            if($focus->object_name == $target_module->object_name){		
			     $target_module->processed = true;
            }
			$target_module->save();
			
		//end for loop of all,first, filter	
		}
	//end if a related record exists to connect this item too.	
	}
//end function_action_new_rel
}


function clean_save_data(& $target_module, $action_array){
	global $app_list_strings;
		foreach($target_module->column_fields as $field){
			if(empty($target_module->$field)){
				
				$data_cleaned = false;
				
				if($target_module->field_defs[$field]['type']=='bool'){
					$target_module->$field = 0;	
					$data_cleaned = true;
				}
					
				if($target_module->field_defs[$field]['type']=='enum'){
					
					$options_array_name = $target_module->field_defs[$field]['options'];						
					$target_module->$field = key($app_list_strings[$options_array_name]);	
					$data_cleaned = true;
				//end if type is enum	
				}	
							
				if($target_module->field_defs[$field]['name']=='duration_hours' ||
					$target_module->field_defs[$field]['name']=='duration_minutes'
				){
					$target_module->$field = '0';	
					$data_cleaned = true;		
				//end if duration hours or minutes from calls module
				}
				
				if(  ( $target_module->field_defs[$field]['name']=='date_start' ||
					$target_module->field_defs[$field]['name']=='time_start' )
					&&
					 (  $target_module->object_name = "Call" ||
						$target_module->object_name = "Meeting"  )
				
				){
					$target_module->$field = get_expiry_date(get_field_type($target_module->field_defs[$field]), 0, false);
					if($target_module->field_defs[$field]['type']=='date' && !empty($target_module->field_defs[$field]['rel_field']) ){
						$rel_field = $target_module->field_defs[$field]['rel_field'];
						$target_module->$rel_field = get_expiry_date('time', $action_array['basic'][$field]);	
					}
					$data_cleaned=true;
				}
							
				if($target_module->field_defs[$field]['name']=='date_entered'){
				
					$data_cleaned=true;
				}		

				if($target_module->field_defs[$field]['name'] == "name"){
				//make sure you set the 'name' to blank, otherwise you won't be able
				//to go into the record
				
					$target_module->$field = " - blank - ";	
					$data_cleaned=true;
				}	
				
				if($data_cleaned==false){		
					
						//try to fill with default if available
						if(!empty($target_module->field_defs[$field]['default'])){
							$target_module->$field = $target_module->field_defs[$field]['default'];
						} else {	
					
							//fill in with blank value
							$target_module->$field = "";
						
						
						}
				}
			
			//end if not empty
			}
			
		//end foreach
		}

//end function clean_save_data
}


	function get_expiry_date($stamp_type, $time_interval, $is_update = false, $value=null){
		
		/* This function needs to be combined with the one in WorkFlowSchedule.php
		Really it should all be moved into the TimeDate stuff. TODO - jgreen.  Contact me 
		with questions.
		
		*/
		
		global $timedate;		
		
		if($is_update == false){
			
			
			$target_stamp = date(dbstampformat($stamp_type));
			//$target_stamp = $timedate->to_display_date_time($target_stamp, true);	
		} else {
			//$target_stamp = $timedate->to_display_date_time($value, true);	
			$target_stamp = $value;
		}		

		//convert stamp and add interval
		$current_unix_stamp = strtotime($target_stamp);	
		$new_unix_stamp = $time_interval + $current_unix_stamp;	
	
		$newtimestamp = gmdate("Y-m-d H:i:s", $new_unix_stamp);

		if($stamp_type=="date"){
			$final_stamp = $timedate->to_display_date($newtimestamp, true);
		}	
		if($stamp_type=="time"){
			$final_stamp = $timedate->to_display_time($newtimestamp, true);
		}	
		if($stamp_type=="datetime"){
			$final_stamp = $timedate->to_display_date_time($newtimestamp, true);	
		}	
	
		return $final_stamp;

	//end function get_expiry_date
	}

	
	function dbstampformat($stamp_type){

		if($stamp_type=="date") return "Y-m-d";
		if($stamp_type=="time") return "H:i:s";
		if($stamp_type=="datetime") return "Y-m-d H:i:s";
	
	}
	
	
?>
