<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Professional Subscription
 * Agreement ("License") which can be viewed at
 * http://www.sugarcrm.com/crm/products/sugar-professional-eula.html
 * By installing or using this file, You have unconditionally agreed to the
 * terms and conditions of the License, and You may not use this file except in
 * compliance with the License.  Under the terms of the license, You shall not,
 * among other things: 1) sublicense, resell, rent, lease, redistribute, assign
 * or otherwise transfer Your rights to the Software, and 2) use the Software
 * for timesharing or service bureau purposes such as hosting the Software for
 * commercial gain and/or for the benefit of a third party.  Use of the Software
 * may be subject to applicable fees and any use of the Software without first
 * paying applicable fees is strictly prohibited.  You do not have the right to
 * remove SugarCRM copyrights from the source code or user interface.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *  (i) the "Powered by SugarCRM" logo and
 *  (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * Your Warranty, Limitations of liability and Indemnity are expressly stated
 * in the License.  Please refer to the License for the specific language
 * governing these rights and limitations under the License.  Portions created
 * by SugarCRM are Copyright (C) 2004-2007 SugarCRM, Inc.; All Rights Reserved.
 ********************************************************************************/
/*********************************************************************************

 * Description:  TODO: To be written.
 * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
 * All Rights Reserved.
 * Contributor(s): ______________________________________..
 ********************************************************************************/

require_once('data/SugarBean.php');
require_once('include/OutboundEmail/OutboundEmail.php');

class Administration extends SugarBean {
	var $settings;
	var $table_name = "config";
	var $object_name = "Administration";
	var $new_schema = true;
	var $module_dir = 'Administration';
	var $config_categories = array(
		// 'mail', // cn: moved to include/OutboundEmail
		'disclosure', // appended to all outbound emails
		'notify', 
		'system',
		'portal',
		'proxy',
		'massemailer',
		'ldap',

		'license',

	);

	var $checkbox_fields = Array("notify_send_by_default", "mail_smtpauth_req", "notify_on", 'portal_on', 'skypeout_on', 'system_mailmerge_on', 'proxy_auth', 'proxy_on', 'system_ldap_enabled');

	function Administration() {
		parent::SugarBean();
		
		 $this->setupCustomFields('Administration');

		$this->disable_row_level_security =true;

	}

	function retrieveSettings($category = FALSE, $clean=false) {
	    // declare a cache for all settings
	    static $settings_cache = array();
	    
	    if($clean) {
	    	$settings_cache = array();	
	    }
	    
	    // Check for a cache hit
	    if(!empty($settings_cache)) {
	        $this->settings = $settings_cache;
	        return $this;
	    }
	    
		$query = "SELECT category, name, value FROM {$this->table_name}";

		$result = $this->db->query($query, true, "Unable to retrieve system settings");
		
		if(empty($result)) {
			return NULL;
		}
		
		while($row = $this->db->fetchByAssoc($result, -1, true)) {
			$this->settings[$row['category']."_".$row['name']] = $row['value'];
		}
		
		// outbound email settings
		$oe = new OutboundEmail();
		$oe->getSystemMailerSettings();
		
		foreach($oe->field_defs as $def) {
			if(strpos($def, "mail_") !== false) 
				$this->settings[$def] = $oe->$def;
		}
		
		// At this point, we have built a new array that should be cached.
        $settings_cache = $this->settings;
		
		return $this;
	}

	function saveConfig() {
		// outbound email settings
		$oe = new OutboundEmail();

		foreach($_POST as $key => $val) {
			$prefix = $this->get_config_prefix($key);

			if(in_array($prefix[0], $this->config_categories)) {
				$this->saveSetting($prefix[0], $prefix[1], $val); 
			}
			if(strpos($key, "mail_") !== false) {
				if(in_array($key, $oe->field_defs)) {
					$oe->$key = $val;
				}
			}
		}

		//saving outbound email from here is probably redundant, adding a check to make sure
		//smtpserver name is set.
		if (!empty($oe->mail_smtpserver)) {
			$oe->saveSystem();
		}
		$this->retrieveSettings(false, true);
	}
	
    function saveSetting($category, $key, $value) {
        $result = $this->db->query("SELECT count(*) AS the_count FROM config WHERE category = '{$category}' AND name = '{$key}'");
        $row = $this->db->fetchByAssoc( $result, -1, true );
        $row_count = $row['the_count'];

        if( $row_count == 0){
            $result = $this->db->query("INSERT INTO config (value, category, name) VALUES ('$value','$category', '$key')");
        }
        else{
            $result = $this->db->query("UPDATE config SET value = '{$value}' WHERE category = '{$category}' AND name = '{$key}'");
        }
        
        return $this->db->getAffectedRowCount($result);
    }

	function get_config_prefix($str) {
		return Array(substr($str, 0, strpos($str, "_")), substr($str, strpos($str, "_")+1));
	}    
}
?>
