<?php
/**
 * The contents of this file are subject to the SugarCRM Professional Subscription
 * Agreement ("License") which can be viewed at
 * http://www.sugarcrm.com/crm/products/sugar-professional-eula.html
 * By installing or using this file, You have unconditionally agreed to the
 * terms and conditions of the License, and You may not use this file except in
 * compliance with the License.  Under the terms of the license, You shall not,
 * among other things: 1) sublicense, resell, rent, lease, redistribute, assign
 * or otherwise transfer Your rights to the Software, and 2) use the Software
 * for timesharing or service bureau purposes such as hosting the Software for
 * commercial gain and/or for the benefit of a third party.  Use of the Software
 * may be subject to applicable fees and any use of the Software without first
 * paying applicable fees is strictly prohibited.  You do not have the right to
 * remove SugarCRM copyrights from the source code or user interface.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *  (i) the "Powered by SugarCRM" logo and
 *  (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * Your Warranty, Limitations of liability and Indemnity are expressly stated
 * in the License.  Please refer to the License for the specific language
 * governing these rights and limitations under the License.  Portions created
 * by SugarCRM are Copyright (C) 2004-2007 SugarCRM, Inc.; All Rights Reserved.
 */



if(!defined('sugarEntry') || !sugarEntry) {
	die('Not A Valid Entry Point');
}

if(!defined('SUGARCRM_SECONDS_PER_DAY')) {
	define('SUGARCRM_SECONDS_PER_DAY', 60 * 60 * 24);
}

require_once ('log4php/LoggerManager.php');

require_once ('data/SugarBean.php');
require_once('include/utils.php');
require_once('modules/ContractTypes/ContractType.php');

class Contract extends SugarBean {
	var $id;
	var $name;
	var $reference_code;
	var $status;
	var $account_id;
	var $account_name;
	var $opportunity_id;
	var $opportunity_name;
	var $team_id;
	var $team_name;
	var $team_link;
	var $assigned_user_id;
	var $assigned_user_name;
	var $assigned_name;
	var $description;
	var $start_date;
	var $end_date;
	var $currency_id;
	var $currency_name;
	var $total_contract_value;
	var $total_contract_value_usdollar;
	var $company_signed_date;
	var $customer_signed_date;
	var $contract_term;
	var $expiration_notice;
	var $time_to_expiry;
	var $date_entered;
	var $date_modified;
	var $deleted;
	var $modified_user_id;
	var $modified_by_name;
	var $created_by;
	var $created_by_name;
	var $created_by_link;
	var $modified_user_link;
	var $assigned_user_link;
	var $contacts;
	var $notes;
	var $products;
	var $quote_id;
	var $type;
	var $type_options;
	
	var $rel_opportunity_table = 'contracts_opportunities';
	var $rel_quote_table = 'contracts_quotes';
	var $table_name = 'contracts';
	var $object_name = 'Contract';
	var $user_preferences;

	var $encodeFields = array ();
	var $relationship_fields = array ('opportunity_id' => 'opportunities', 'note_id' => 'notes', 'quote_id' => 'quotes',);
	// This is used to retrieve related fields from form posts.
	var $additional_column_fields = array ('revision');

	

	var $new_schema = true;
	var $module_dir = 'Contracts';

	function Contract() {
		parent :: SugarBean();
		$this->setupCustomFields('Contract'); //parameter is module name
		$this->disable_row_level_security = false;
	}

	function save($check_notify = false) {
		//decimals cant be null in sql server	
		if ( $this->total_contract_value == '' ) { $this->total_contract_value = '0'; }	
		if ( $this->total_contract_value_usdollar == '' ) { $this->total_contract_value_usdollar = '0'; }		
		return parent :: save($check_notify);
	}

	function get_summary_text() {
		return $this->name;
	}

	function retrieve($id, $encode = false) {
		$ret = parent :: retrieve($id, $encode);
		return $ret;
	}

	function is_authenticated() {
		return $this->authenticated;
	}

	function fill_in_additional_list_fields() {
		$this->fill_in_additional_detail_fields();
	}

	function _set_related_opportunity_info()
	{
		$contracts_table = $this->table_name;
		$opportunity_link_table = $this->rel_opportunity_table;
		$query = "SELECT opportunities.id, opportunities.name "
			. "FROM opportunities "
			. "INNER JOIN $opportunity_link_table "
			. "ON opportunities.id=$opportunity_link_table.opportunity_id "
			. "INNER JOIN $contracts_table "
			. "ON $contracts_table.id=$opportunity_link_table.contract_id "
			. "WHERE contracts.id='$this->id' AND contracts.deleted=0 AND opportunities.deleted=0 AND $opportunity_link_table.deleted=0 ";
		$result = $this->db->query($query, true, 'Error retrieving opportunity info for contract: ');
		$row = $this->db->fetchByAssoc($result);

		if(!empty($row)) {
			$this->opportunity_id = stripslashes($row['id']);
			$this->opportunity_name = stripslashes($row['name']);
		}
	}
	
	function _set_related_account_info() {
		$contracts_table = $this->table_name;
		
		if(!empty($this->account_id)) {
			$query = "SELECT accounts.name "
				. "FROM accounts "
				. "INNER JOIN $contracts_table "
				. "ON accounts.id=$contracts_table.account_id "
				. "WHERE $contracts_table.id='$this->id' AND $contracts_table.deleted=0 AND accounts.deleted=0 ";
			$name = $this->db->getOne($query, true, 'Error retrieving account info for contract: ');
	
			if(!empty($name)) {
				$this->account_name = stripslashes($name);
			}
		}
	}

	function _set_contract_term() {
		global $timedate;
		$start_date_timestamp = empty($this->start_date) ? 0 : strtotime($timedate->to_db_date($this->start_date, false));
		$end_date_timestamp = empty($this->end_date) ? 0 : strtotime($timedate->to_db_date($this->end_date, false));
		$this->contract_term = '';
		if(!empty($start_date_timestamp) && !empty($end_date_timestamp)) {
			$this->contract_term = ($end_date_timestamp - $start_date_timestamp) / constant('SUGARCRM_SECONDS_PER_DAY');
		}
	}		
	function _set_time_to_expiry() {
		global $timedate;
		$end_date_timestamp = empty($this->end_date) ? 0 : strtotime($timedate->to_db_date($this->end_date, false));
		$now = time();
		$this->time_to_expiry = '';
		if(!empty($end_date_timestamp)) {
			$this->time_to_expiry = ($end_date_timestamp - $now) / constant('SUGARCRM_SECONDS_PER_DAY');
		}
	}

	function fill_in_additional_detail_fields() {
		parent::fill_in_additional_detail_fields();
		$this->_set_related_account_info();
		$this->_set_related_opportunity_info();
		$this->_set_contract_term();
		$this->_set_time_to_expiry();

		$types = get_bean_select_array(true, 'ContractType','name','deleted=0','list_order');
		$this->type_options = get_select_options_with_id($types, $this->type);	
        $currency= new Currency();
        
		if(isset($this->currency_id) && !empty($this->currency_id)) {
			$currency->retrieve($this->currency_id);
		
			if( $currency->deleted != 1) {
				$this->currency_name = $currency->iso4217 . ' ' . $currency->symbol;
			} else {
				$this->currency_name = $currency->getDefaultISO4217() . ' ' . $currency->getDefaultCurrencySymbol();
			}
		} else {
		    $this->currency_name = $currency->getDefaultISO4217() . ' ' . $currency->getDefaultCurrencySymbol();
		}
	}

	function list_view_parse_additional_sections(& $list_form, $xTemplateSection) {
		return $list_form;
	}

	function create_export_query($order_by, $where) {
		$query = 'SELECT contracts.* FROM contracts ';

		$where_auto = ' contracts.deleted = 0';

		if (empty($where)) {
			$query .= " WHERE $where_auto";
		} else {
			$query .= " WHERE $where AND $where_auto";
		}

		if (empty($order_by)) {
			$query .= ' ORDER BY contracts.name';
		} else {
			$query .= " ORDER BY $order_by";
		}

		return $query;
	}

	function create_list_query($order_by, $where, $show_deleted = 0) {
		$custom_join = false;
		$table_name = $this->table_name;
		
		if (isset ($this->custom_fields)) {
			$custom_join = $this->custom_fields->getJOIN();
		}

		$query = "SELECT accounts.name account_name, "
			. "users.user_name assigned_user_name, "
			. "teams.name team_name, "
			. "$table_name.* ";

		if ($custom_join) {
			$query .= $custom_join['select'];
		}

		$query .= " FROM $table_name ";

		$this->add_team_security_where_clause($query);

		$query .= " LEFT JOIN accounts "
			. "ON contracts.account_id=accounts.id ";
		$query .= "LEFT JOIN users "
			. "ON contracts.assigned_user_id=users.id ";
		$query .= "LEFT JOIN teams "
			. "ON contracts.team_id=teams.id ";
		

		if ($custom_join) {
			$query .= $custom_join['join'];
		}

		$where_auto = '1=1';
		if ($show_deleted == 0) {
			$where_auto = " $table_name.deleted=0 ";
		} elseif ($show_deleted == 1) {
			$where_auto = " $table_name.deleted=1 ";
		}
		
		if ($where != '') {
			$query .= " WHERE ($where) AND ";
		} else {
			$query .= " WHERE ";
		}

		$query .= " $table_name.deleted=0 ";

		if (!empty ($order_by)) {
			$query .= " ORDER BY $order_by ";
		}

		return $query;
	}

	function get_list_view_data() {
		$fields = $this->get_list_view_array();
		require_once('modules/Currencies/Currency.php');
		$fields['TOTAL_CONTRACT_VALUE']= format_number($fields['TOTAL_CONTRACT_VALUE']);
		return $fields;
	}

	function mark_relationships_deleted($id) {
		// Do nothing, this call is here to avoid default delete processing
		// since Delete.php handles deletion of children records.
	}
	
	function bean_implements($interface) {
		$ret_val = false;
		
		switch($interface) {
			case 'ACL':
				$ret_val = true;
				break;
		}
		
		return $ret_val;
	}
	
	function listviewACLHelper() {
		global $current_user;
		$array_assign = parent::listviewACLHelper();
		$is_owner = false;
		
		if(!empty($this->account_id)) {
			if(!empty($this->account_id_owner)) {
				global $current_user;
				$is_owner = ($current_user->id == $this->account_id_owner);
			}
		}
		
		if(!ACLController::moduleSupportsACL('Accounts') || ACLController::checkAccess('Accounts', 'view', $is_owner)) {
			$array_assign['ACCOUNT'] = 'a';
		} else {
			$array_assign['ACCOUNT'] = 'span';
		}
		
		if(!empty($this->opportunity_name)) {
			if(!empty($this->opportunity_name_owner)) {
				global $current_user;
				$is_owner = $current_user->id == $this->opportunity_name_owner;
			}
		}

		if(!ACLController::moduleSupportsACL('Opportunities') || ACLController::checkAccess('Opportunities', 'view', $is_owner)) {
			$array_assign['OPPORTUNITY'] = 'a';
		} else {
			$array_assign['OPPORTUNITY'] = 'span';
		}
		
		return $array_assign;
	}
	
	function get_contract_documents() {
		$this->load_relationship('documents');
		$query_array = $this->documents->getQuery(true);		
		$query = <<<KGB
			SELECT documents.*,
				documents.document_revision_id AS latest_revision_id,
				for_latest_revision.revision AS latest_revision_name,
				for_selected_revision.revision AS selected_revision_name,
				linked_documents.document_revision_id AS selected_revision_id,
				contracts.status AS contract_status,
				for_selected_revision.filename AS selected_revision_filename,
				linked_documents.id AS linked_id,
				contracts.name AS contract_name

KGB;
					
		$query .= $query_array['from'];
		$query .= <<<CIA
			LEFT JOIN document_revisions for_latest_revision 
				ON for_latest_revision.id = documents.document_revision_id
			INNER JOIN contracts 
				ON contracts.id = linked_documents.parent_id
			LEFT JOIN document_revisions for_selected_revision 
				ON for_selected_revision.id = linked_documents.document_revision_id

CIA;
		$query .= $query_array['where'];
		return $query;
	}
	
}

function getContractTypesDropDown(){		
		static $contractTypes = null;
		if(!$contractTypes){
			$seedContractTypes = new ContractType();
			$contractTypes = $seedContractTypes->get_contractTypes(TRUE);
		}		
		return $contractTypes;
	}
?>
