/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Professional Subscription
 * Agreement ("License") which can be viewed at
 * http://www.sugarcrm.com/crm/products/sugar-professional-eula.html
 * By installing or using this file, You have unconditionally agreed to the
 * terms and conditions of the License, and You may not use this file except in
 * compliance with the License.  Under the terms of the license, You shall not,
 * among other things: 1) sublicense, resell, rent, lease, redistribute, assign
 * or otherwise transfer Your rights to the Software, and 2) use the Software
 * for timesharing or service bureau purposes such as hosting the Software for
 * commercial gain and/or for the benefit of a third party.  Use of the Software
 * may be subject to applicable fees and any use of the Software without first
 * paying applicable fees is strictly prohibited.  You do not have the right to
 * remove SugarCRM copyrights from the source code or user interface.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *  (i) the "Powered by SugarCRM" logo and
 *  (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * Your Warranty, Limitations of liability and Indemnity are expressly stated
 * in the License.  Please refer to the License for the specific language
 * governing these rights and limitations under the License.  Portions created
 * by SugarCRM are Copyright (C) 2004-2007 SugarCRM, Inc.; All Rights Reserved.
 ********************************************************************************/
/**
  Complex layout init
 */
function complexLayoutInit() {
    e2Layout = function(){
        //var mainPanel;

        return {
            getComposeLayout : function() {
                var idx = SUGAR.email2.composeLayout.currentInstanceId;
                SUGAR.email2.composeLayout[idx] = new Ext.BorderLayout('composeLayout' + idx, {
                    center:{
                        preservePanel : false,
                        animate: false,
                        autoScroll: false,
                        split:true,
                        tabPosition: SUGAR.email2.userPrefs.emailSettings.tabPosition,
                        titlebar: false
                    },
                    east:{
                        preservePanel : false,
                        animate: false,
                        autoScroll: true,
                        autoTab: true,
                        collapsible: true,
                        collapsed: true,
                        split:true,
                        tabPosition: SUGAR.email2.userPrefs.emailSettings.tabPosition,
                        initialSize: 300,
                        titlebar: true,
                        shim: true,
                        iframe: true
                    }
                });
                SUGAR.email2.composeLayout[idx].add('east', new Ext.ContentPanel('divAttachments' + idx, {title: app_strings.LBL_EMAIL_ATTACHMENT, closable: false}));
                SUGAR.email2.composeLayout[idx].add('east', new Ext.ContentPanel('divOptions' + idx, {title: app_strings.LBL_EMAIL_OPTIONS, closable: false}));
                SUGAR.email2.composeLayout[idx].add('center', new Ext.ContentPanel('composeOverFrame' + idx));
                //Convert selects to combo boxes for IE compatibility with panels

                
//                SUGAR.email2.composeLayout[idx].regions.east.expand();
//                SUGAR.email2.composeLayout[idx].regions.east.collapse();
            },
            
            getInnerLayout : function(rows) {
             SUGAR.email2.listViewLayout = new Ext.BorderLayout('listViewLayout', {
                    center: {
                        autoScroll:false, // grid should autoScroll itself
                        initialSize:'250',
                        titlebar:false,
                        split:true
                    },
                    south: {
                        autoScroll:true,
                        collapsible: true,
                        initialSize:'350',
                        titlebar: false,
                        split: true
                    },
                    east: {
                        autoScroll:true,
                        collapsible: false,
                        initialSize:'400',
                        titlebar: false,
                        split: true
                    }
                });
                SUGAR.email2.listViewLayout.beginUpdate();
                if (!SUGAR.email2.e2gridPanel){
                    SUGAR.email2.e2gridPanel = new Ext.GridPanel(SUGAR.email2.grid, {closable: false});
                }
                SUGAR.email2.listViewLayout.add('center', SUGAR.email2.e2gridPanel);
                SUGAR.email2.listViewLayout.add(rows ? 'south' : 'east', new Ext.ContentPanel('_blank', {closable: false}));
                SUGAR.email2.listViewLayout.restoreState();
                SUGAR.email2.listViewLayout.endUpdate(false);
                SUGAR.email2.listViewLayout.getRegion(rows ? 'east' : 'south').hide();
                
                SUGAR.email2.complexLayout.getRegion('center').remove('innerLayout');
                SUGAR.email2.innerLayout = new Ext.BorderLayout('innerLayout', {
                    center: {
                        autoScroll:true,// for popup msgs
                        titlebar:true,
                        autoTab: true,
                        closeOnTab: true,
                        alwaysShowTabs: true,
                        resizeTabs: true,
                        tabPosition: SUGAR.email2.userPrefs.emailSettings.tabPosition
                    }
                });

                SUGAR.email2.innerLayout.beginUpdate();
                var centerPanel = new Ext.NestedLayoutPanel(SUGAR.email2.listViewLayout);
                centerPanel.setTitle('&nbsp;');
                SUGAR.email2.innerLayout.add('center', centerPanel);
                SUGAR.email2.innerLayout.endUpdate();
                // set preview layout pointer
                SUGAR.email2.previewLayout = SUGAR.email2.listViewLayout;
                SUGAR.email2.complexLayout.add('center', new Ext.NestedLayoutPanel(SUGAR.email2.innerLayout));
            },
            
            getInnerLayout2Rows : function() {
                this.getInnerLayout(true);
            },
            getInnerLayout2Columns : function() {
                this.getInnerLayout(false);
            },
            
            init : function(){
                // initialize state manager, we will use cookies
                Ext.state.Manager.setProvider(new Ext.state.CookieProvider());

                SUGAR.email2.complexLayout = new Ext.BorderLayout("container", {
                    hideOnLayout: true,
                    center: {
                        preservePanel : false,
                        animate: false,
                        autoScroll:false,
                        tabPosition: SUGAR.email2.userPrefs.emailSettings.tabPosition,
                        collapsible: false,
                        titlebar: false
                    },
                    west: {
                        animate: false,
                        autoScroll: true,
                        collapsible: true,
                        initialSize: 300,
                        minSize: 100,
                        split:true,
                        tabPosition: SUGAR.email2.userPrefs.emailSettings.tabPosition,
                        useShim:true,
                        titlebar: true
                    },
                    south: {
                        collapsible: false,
                        collapsed: false,
                        initialSize: 25,
                        split:false,
                        titlebar: true
                    }
                });
    
                SUGAR.email2.complexLayout.beginUpdate();
                    SUGAR.email2.userPrefs.emailSettings.layoutStyle == '2rows' ? this.getInnerLayout2Rows() : this.getInnerLayout2Columns();
                    
                    var innerLayout = new Ext.NestedLayoutPanel(SUGAR.email2.innerLayout);
                    var contacts = new Ext.ContentPanel('contactsTab', {
                       title: app_strings.LBL_EMAIL_ADDRESS_BOOK_TITLE_ICON, 
                       autoScroll: true
                    });
                    var searchContentPanel = new Ext.ContentPanel('searchTab', {title: app_strings.LBL_EMAIL_SEARCH});
                    SUGAR.email2.complexLayout.add('center', innerLayout);
                    SUGAR.email2.complexLayout.add('west', new Ext.ContentPanel('frameFolders', {title: app_strings.LBL_EMAIL_FOLDERS}));
                    SUGAR.email2.complexLayout.add('west', searchContentPanel);
                    SUGAR.email2.complexLayout.add('west', contacts);



                    SUGAR.email2.complexLayout.add('south', new Ext.ContentPanel('footerLinks', {title: document.getElementById('footerLinks').innerHTML}));
                    SUGAR.email2.complexLayout.showPanel('frameFolders');
                    SUGAR.email2.complexLayout.restoreState();
                SUGAR.email2.complexLayout.endUpdate();
                
                contacts.addListener('activate', SUGAR.email2.addressBook.getUserContacts);
                searchContentPanel.addListener('activate', SUGAR.email2.search.updateSearchTab);
                
                //IE Fails to redraw when switching panels, this forces its hand.
                if (SUGAR.email2.util.isIe()) {
                    SUGAR.email2.innerLayout.regions.center.on('panelactivated', function(panel) { 
                        SUGAR.email2.innerLayout.regions.center.resizeTo(100);
                    } );
                }
                
                // ear-mark the layout to target dblclick'd emails
                SUGAR.email2.innerLayout.regions.center.addListener("beforeremove", SUGAR.email2.composeLayout.confirmClose);
            }
        }
    }();
    e2Layout.init();


    SUGAR.email2.complexLayout.toggleArea = function(id){
        var area = SUGAR.email2.complexLayout.getRegion(id);
        if(area.isVisible()){
            area.hide();
        } else {
            area.show();
        }
    }

    SUGAR.email2.innerLayout.toggleArea = function(id){
        var area = SUGAR.email2.innerLayout.getRegion(id);
        if(area.isVisible()){
            area.hide();
        } else {
            area.show();
        }
    }
    
    SUGAR.email2.autoSetLayout();
}

var myBufferedListenerObject = new Object();
myBufferedListenerObject.refit = function() {
    if(SUGAR.email2.grid) {
        SUGAR.email2.grid.autoSize();
    }
}
