<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Professional Subscription
 * Agreement ("License") which can be viewed at
 * http://www.sugarcrm.com/crm/products/sugar-professional-eula.html
 * By installing or using this file, You have unconditionally agreed to the
 * terms and conditions of the License, and You may not use this file except in
 * compliance with the License.  Under the terms of the license, You shall not,
 * among other things: 1) sublicense, resell, rent, lease, redistribute, assign
 * or otherwise transfer Your rights to the Software, and 2) use the Software
 * for timesharing or service bureau purposes such as hosting the Software for
 * commercial gain and/or for the benefit of a third party.  Use of the Software
 * may be subject to applicable fees and any use of the Software without first
 * paying applicable fees is strictly prohibited.  You do not have the right to
 * remove SugarCRM copyrights from the source code or user interface.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *  (i) the "Powered by SugarCRM" logo and
 *  (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * Your Warranty, Limitations of liability and Indemnity are expressly stated
 * in the License.  Please refer to the License for the specific language
 * governing these rights and limitations under the License.  Portions created
 * by SugarCRM are Copyright (C) 2004-2007 SugarCRM, Inc.; All Rights Reserved.
 ********************************************************************************/
/*********************************************************************************

 * Description:  Defines the Account SugarBean Account entity with the necessary
 * methods and variables.
 ********************************************************************************/



require_once('data/SugarBean.php');
require_once('modules/Contacts/Contact.php');
require_once('modules/Opportunities/Opportunity.php');
require_once('modules/Cases/Case.php');
require_once('modules/Calls/Call.php');
require_once('modules/Notes/Note.php');
require_once('modules/Emails/Email.php');
require_once('modules/Leads/Lead.php');

global $app_list_strings;

class ImportLead extends Lead {
	 var $db;

	// these are fields that may be set on import
	// but are to be processed and incorporated
	// into fields of the parent class


	// This is the list of fields that are required.
	var $required_fields =  array("last_name"=>1);
	
	// This is the list of the functions to run when importing
	var $special_functions =  array(
		"get_names_from_full_name"
                ,"add_salutation"
                ,"add_lead_status"
                ,"add_lead_source"
                ,"add_do_not_call"
                ,"add_email_opt_out"
		,"add_primary_address_streets"
		,"add_alt_address_streets"
	 );


		//removed importable_fields, this array is now generated in the import wizard. and The array is based
		//on the meta defined in the vardef file for the leads module.

        function add_salutation()
        {
                global $app_list_strings;
                if ( isset($this->salutation) &&
                        ! isset( $app_list_strings['salutation_dom'][ $this->salutation ]) )
                {
                        $this->salutation = '';
                }
        }

        function add_lead_source()
        {
                global $app_list_strings;
                if ( isset($this->lead_source) &&
                        ! isset( $app_list_strings['lead_source_dom'][ $this->lead_source ]) )
                {
                        $this->lead_source = '';
                }

        }

        function add_lead_status()
        {
                global $app_list_strings;
                if ( isset($this->status) &&
                        ! isset( $app_list_strings['lead_status_dom'][ $this->status ]) )
                {
                        $this->status = '';
                }

        }

        function add_do_not_call()
        {
			if ( isset($this->do_not_call) && strtoupper($this->do_not_call) == 'ON')
			{
				$this->do_not_call = 1;
			}
			else {
				$this->do_not_call=0;
			}
        }

        function add_email_opt_out()
        {
                if ( isset($this->email_opt_out) && ($this->email_opt_out != 1 ))
                {
                        $this->email_opt_out = '';
                }
        }

        function get_names_from_full_name()
        {
                if ( ! isset($this->full_name))
                {
                        return;
                }
                $arr = array();

                $name_arr = preg_split('/\s+/',$this->full_name);

                if ( count($name_arr) == 1)
                {
                        $this->last_name = $this->full_name;
                }
                else
                {
                        $this->first_name = array_shift($name_arr);

                        $this->last_name = join(' ',$name_arr);
                }

        }
        function add_primary_address_streets()
        {
                if ( isset($this->primary_address_street_2))
                {
                        $this->primary_address_street .= " ". $this->primary_address_street_2;
                }

                if ( isset($this->primary_address_street_3))
                {
                        $this->primary_address_street .= " ". $this->primary_address_street_3;
                }
        }

        function add_alt_address_streets()
        {
                if ( isset($this->alt_address_street_2))
                {
                        $this->alt_address_street .= " ". $this->alt_address_street_2;
                }

                if ( isset($this->alt_address_street_3))
                {
                        $this->alt_address_street .= " ". $this->alt_address_street_3;
                }

        }


	//module prefix used by ImportSteplast when calling ListView.php
	var $list_view_prefix = 'LEAD';

	//columns to be displayed in listview for displaying user's last import in ImportSteplast.php
	var $list_fields = Array(
					  'id'
					, 'first_name'
					, 'last_name'
					, 'account_name'
					, 'title'
					, 'phone_work'
					, 'assigned_user_name'
					, 'assigned_user_id'
					, 'lead_source'
					, 'lead_source_description'
					, 'refered_by'
					, 'opportunity_name'
					, 'opportunity_amount'
					, 'date_entered'
					, 'status'
					);

	//this list defines what beans get populated during an import of this leads
	var $related_modules = array("Leads",); 
		

	function ImportLead() {
		parent::Lead();
	}

	function create_list_query($order_by, $where, $show_deleted = 0) {
		global $current_user;
		$query = "SELECT
			leads.*,
			users.user_name as assigned_user_name ";

		$query .= ", teams.name as team_name ";

		$query .= "FROM users_last_import,leads LEFT JOIN users ON leads.assigned_user_id=users.id";

		$query .= " JOIN teams ON leads.team_id = teams.id ";

		$query .= " WHERE users_last_import.assigned_user_id='{$current_user->id}'
			AND users_last_import.bean_type='Leads'
			AND users_last_import.bean_id=leads.id
			AND users_last_import.deleted=0
			AND leads.deleted=0 ";

		if(!empty($order_by)) {
			$query .= " ORDER BY {$order_by}";
		}

		return $query;
	}
}



?>
