<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Professional Subscription
 * Agreement ("License") which can be viewed at
 * http://www.sugarcrm.com/crm/products/sugar-professional-eula.html
 * By installing or using this file, You have unconditionally agreed to the
 * terms and conditions of the License, and You may not use this file except in
 * compliance with the License.  Under the terms of the license, You shall not,
 * among other things: 1) sublicense, resell, rent, lease, redistribute, assign
 * or otherwise transfer Your rights to the Software, and 2) use the Software
 * for timesharing or service bureau purposes such as hosting the Software for
 * commercial gain and/or for the benefit of a third party.  Use of the Software
 * may be subject to applicable fees and any use of the Software without first
 * paying applicable fees is strictly prohibited.  You do not have the right to
 * remove SugarCRM copyrights from the source code or user interface.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *  (i) the "Powered by SugarCRM" logo and
 *  (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * Your Warranty, Limitations of liability and Indemnity are expressly stated
 * in the License.  Please refer to the License for the specific language
 * governing these rights and limitations under the License.  Portions created
 * by SugarCRM are Copyright (C) 2004-2007 SugarCRM, Inc.; All Rights Reserved.
 ********************************************************************************/
/*********************************************************************************

 * Description:  TODO: To be written.
 ********************************************************************************/



require_once('modules/Manufacturers/Manufacturer.php');
require_once('modules/ProductTemplates/ProductTemplate.php');
require_once('modules/ProductCategories/ProductCategory.php');
require_once('modules/ProductTypes/ProductType.php');
require_once('modules/Import/UsersLastImport.php');

global $app_list_strings;

class ImportProductTemplate extends ProductTemplate {
	// these are fields that may be set on import
	// but are to be processed and incorporated
	// into fields of the parent class
	var $db;

	// This is the list of fields that are required.
	var $required_fields =  array(
				"name"=>1,
				"cost_price"=>1,
				"discount_price"=>1,
				"list_price"=>1);

       // This is the list of the functions to run when importing
        var $special_functions =  array(
        "add_manufacturer",
        "add_cost_price",
        "add_discount_price",
        "add_list_price",
        "add_weight",
        "add_quantity",
        "add_pricing_formula",
        "add_pricing_factor",
        "add_tax_class",
        "add_status",
        "add_date_available",
        "add_support_term",
        "add_category",
        "add_type",
        "add_currency_name",
	);

	function add_currency_name() {
		
		//trigger this only if currenct_id is not set and currency  name is set.
		//try to locate currency by name.
		//if found set the currency_id else set default currency id.
		if (!empty($this->currency) && empty($this->currency_id)) {
			require_once('modules/Currencies/Currency.php');
			$currency = new Currency();
			$currency_id=$currency->retrieve_id_by_name($this->currency);
			//empty $currency_id, set a default value...
			if (empty($currency_id)) $currency_id=-99;
			$this->currency_id = $currency_id;
		}
	}
	function add_cost_price()
        {
                if ( isset($this->cost_price) &&
			! is_numeric($this->cost_price) )
		{
			$this->cost_price = '';
                }
        }
	function add_discount_price()
        {
                if ( isset($this->discount_price) &&
			! is_numeric($this->discount_price) )
		{
			$this->discount_price = '';
                }
        }

	function add_list_price()
        {
                if ( isset($this->list_price) &&
			! is_numeric($this->list_price) )
		{
			$this->list_price = '';
                }
        }

	function add_date_available()
        {
                if ( isset($this->date_available))
                {
                        if (! preg_match('/^\d{4}-\d{1,2}-\d{1,2}$/',$this->date_available))
                        {
                                $this->date_available = '';
                        }
                }

        }


	function add_weight()
        {
                if ( isset($this->weight) &&
			! is_numeric($this->weight) )
		{
			$this->weight = '';
                }
        }

	function add_quantity()
        {
                if ( isset($this->quantity) &&
			! is_numeric($this->quantity) )
		{
			$this->quantity = '';
                }
        }

	function add_status()
        {
                if ( isset($this->status) 
			&& $this->status != 'Available'
			&& $this->status != 'Unavailable' )
		{
			$this->status = 'Available';
		}
        }

	function add_support_term()
        {
                global $app_list_strings;

                if ( isset($this->support_term) &&
                        ! isset( $app_list_strings['support_term_dom'][ $this->support_term ]) )
                {
                        $this->support_term = '';
                }
        }

	function add_tax_class()
        {
                global $app_list_strings;

                if ( isset($this->tax_class) &&
                        ! isset( $app_list_strings['tax_class_dom'][ $this->tax_class ]) )
                {
                        $this->tax_class = '';
                }
        }

	function add_pricing_factor()
        {
                if ( isset($this->pricing_factor) &&
                	! is_numeric($this->pricing_factor) )
                {
                        $this->pricing_factor = '';
                }
        }

	function add_pricing_formula()
        {
                global $app_list_strings;

                if ( isset($this->pricing_formula) &&
                        ! isset( $app_list_strings['pricing_formula_dom'][ $this->pricing_formula ]) )
                {
                        $this->pricing_formula = '';
                }
        }

        function add_type()
        {
		global $current_user;

		if ( empty($this->type_name) &&  empty($this->type_id))
		{
			return;
		}

                $arr = array();

		// check if it already exists
                $focus = new ProductType();

		$query = '';

		// if user is defining the type id to be associated with this product..
		if ( isset($this->type_id) && $this->type_id != '')
		{
                	$query = "select * from {$focus->table_name} WHERE id='". PearDatabase::quote($this->type_id)."'";
		}
		// else user is defining the type name to be associated with this contact..
		else
		{
                	$query = "select * from {$focus->table_name} WHERE name='". PearDatabase::quote($this->type_name)."'";
		}

                $GLOBALS['log']->info($query);

                $result = $this->db->query($query)
                       or sugar_die("Error selecting sugarbean: ");

                $row = $this->db->fetchByAssoc($result, -1, false);

		// we found a row with that id
                if (isset($row['id']) && $row['id'] != -1)
                {
                        // if it exists but was deleted, just remove it entirely
                        if ( isset($row['deleted']) && $row['deleted'] == 1)
                        {
                                $query2 = "delete from {$focus->table_name} WHERE id='". PearDatabase::quote($row['id'])."'";

                                $GLOBALS['log']->info($query2);

                                $result2 = $this->db->query($query2)
                                        or sugar_die("Error deleting existing sugarbean: ");

                        }
			// else just use this id to link the product to the type
                        else
                        {
                                $focus->id = $row['id'];
                        }
                }

		// if we didnt find the type, so create it
                if (! isset($focus->id) || $focus->id == '')
                {
                        $focus->name = $this->type_name;
                        $focus->assigned_user_id = $current_user->id;
                        $focus->modified_user_id = $current_user->id;

			// if we are providing the type id:
			if ( isset($this->type_id)  &&
                                $this->type_id != '')
                        {
				$focus->new_with_id = true;
                                $focus->id = $this->type_id;
                        }

                        $focus->save();
			// avoid duplicate mappings:
			if (! isset( $imported_ids[$focus->id]) )
			{
				// save the new account as a users_last_import
                		$last_import = new UsersLastImport();
                		$last_import->assigned_user_id = $current_user->id;
                		$last_import->bean_type = "ProductTypes";
                		$last_import->bean_id = $focus->id;
                		$last_import->save();
				$imported_ids[$focus->id] = 1;
			}
                }

		// now just link the type
                $this->type_id = $focus->id;

        }

        function add_category()
        {
		global $current_user;

		if ( empty($this->category_name) &&  empty($this->category_id))
		{
			return;
		}

                $arr = array();

		// check if it already exists
                $focus = new ProductCategory();

		$query = '';

		// if user is defining the category id to be associated with this contact..
		if ( isset($this->category_id) && $this->category_id != '')
		{
                	$query = "select * from {$focus->table_name} WHERE id='". PearDatabase::quote($this->category_id)."'";
		}
		// else user is defining the category name to be associated with this product..
		else
		{
                	$query = "select * from {$focus->table_name} WHERE name='". PearDatabase::quote($this->category_name)."'";
		}

                $GLOBALS['log']->info($query);

                $result = $this->db->query($query)
                       or sugar_die("Error selecting sugarbean: ");

                $row = $this->db->fetchByAssoc($result, -1, false);

		// we found a row with that id
                if (isset($row['id']) && $row['id'] != -1)
                {
                        // if it exists but was deleted, just remove it entirely
                        if ( isset($row['deleted']) && $row['deleted'] == 1)
                        {
                                $query2 = "delete from {$focus->table_name} WHERE id='". PearDatabase::quote($row['id'])."'";

                                $GLOBALS['log']->info($query2);

                                $result2 = $this->db->query($query2)
                                        or sugar_die("Error deleting existing sugarbean: ");

                        }
			// else just use this id to link the lead to the category 
                        else
                        {
                                $focus->id = $row['id'];
                        }
                }

		// if we didnt find the category, so create it
                if (! isset($focus->id) || $focus->id == '')
                {
                        $focus->name = $this->category_name;
                        $focus->assigned_user_id = $current_user->id;
                        $focus->modified_user_id = $current_user->id;

			// if we are providing the category id:
			if ( isset($this->category_id)  &&
                                $this->category_id != '')
                        {
				$focus->new_with_id = true;
                                $focus->id = $this->category_id;
                        }

                        $focus->save();
			// avoid duplicate mappings:
			if (! isset( $imported_ids[$focus->id]) )
			{
				// save the new account as a users_last_import
                		$last_import = new UsersLastImport();
                		$last_import->assigned_user_id = $current_user->id;
                		$last_import->bean_type = "ProductCategories";
                		$last_import->bean_id = $focus->id;
                		$last_import->save();
				$imported_ids[$focus->id] = 1;
			}
                }

		// now just link the category
                $this->category_id = $focus->id;

        }

        function add_manufacturer()
        {
		global $current_user;

		if ( empty($this->manufacturer_name) &&  empty($this->manufacturer_id))
		{
			return;
		}

                $arr = array();

		// check if it already exists
                $focus = new Manufacturer();

		$query = '';

		// if user is defining the manufacturer id to be associated with this contact..
		if ( isset($this->manufacturer_id) && $this->manufacturer_id != '')
		{
                	$query = "select * from {$focus->table_name} WHERE id='". PearDatabase::quote($this->manufacturer_id)."'";
		}
		// else user is defining the manufacturer name to be associated with this contact..
		else
		{
                	$query = "select * from {$focus->table_name} WHERE name='". PearDatabase::quote($this->manufacturer_name)."'";
		}

                $GLOBALS['log']->info($query);

                $result = $this->db->query($query)
                       or sugar_die("Error selecting sugarbean: ");

                $row = $this->db->fetchByAssoc($result, -1, false);

		// we found a row with that id
                if (isset($row['id']) && $row['id'] != -1)
                {
                        // if it exists but was deleted, just remove it entirely
                        if ( isset($row['deleted']) && $row['deleted'] == 1)
                        {
                                $query2 = "delete from {$focus->table_name} WHERE id='". PearDatabase::quote($row['id'])."'";

                                $GLOBALS['log']->info($query2);

                                $result2 = $this->db->query($query2)
                                        or sugar_die("Error deleting existing sugarbean: ");

                        }
			// else just use this id to link the contact to the manufacturer
                        else
                        {
                                $focus->id = $row['id'];
                        }
                }

		// if we didnt find the manufacturer, so create it
                if (! isset($focus->id) || $focus->id == '')
                {
                        $focus->name = $this->manufacturer_name;
                        $focus->assigned_user_id = $current_user->id;
                        $focus->modified_user_id = $current_user->id;

			// if we are providing the manufacturer id:
			if ( isset($this->manufacturer_id)  &&
                                $this->manufacturer_id != '')
                        {
				$focus->new_with_id = true;
                                $focus->id = $this->manufacturer_id;
                        }

                        $focus->save();
			// avoid duplicate mappings:
			if (! isset( $imported_ids[$focus->id]) )
			{
				// save the new account as a users_last_import
                		$last_import = new UsersLastImport();
                		$last_import->assigned_user_id = $current_user->id;
                		$last_import->bean_type = "Manufacturers";
                		$last_import->bean_id = $focus->id;
                		$last_import->save();
				$imported_ids[$focus->id] = 1;
			}
                }

		// now just link the manufacturer
                $this->manufacturer_id = $focus->id;

        }

	//module prefix used by ImportSteplast when calling ListView.php
	var $list_view_prefix = 'PRODUCT_TEMPLATE';

	//columns to be displayed in listview for displaying user's last import in ImportSteplast.php
	var $list_fields = Array(
					  'id'
					, 'name'
					, 'status'
					, 'qty_in_stock'
					, 'cost_price'
					, 'cost_usdollar'
					, 'discount_price'
					, 'discount_usdollar'
					, 'list_price'
					, 'list_usdollar'
					, 'mft_part_num'
					, 'pricing_factor'
					, 'pricing_formula'
					, 'tax_class'
					, 'type_name'
					, 'type_id'
					, 'category_name'
					, 'category_id');
					
	//this list defines what beans get populated during an import of product templates
	var $related_modules = array("ProductTemplates","ProductCategories","ProductTypes","Manufacturers"); 
		

	function ImportProductTemplate() {
		parent::ProductTemplate();
	}

	function create_list_query($order_by, $where, $show_deleted = 0)
	{
		global $current_user;
		$query = '';
        $query = "SELECT $this->table_name.*,
				$this->rel_manufacturers.name as manufacturer_name,
                                $this->rel_categories.name as category_name,
                                $this->rel_types.name as type_name
                                FROM users_last_import,$this->table_name
                                LEFT JOIN $this->rel_manufacturers
                                ON $this->table_name.manufacturer_id=$this->rel_manufacturers.id AND $this->rel_manufacturers.deleted=0
                                LEFT JOIN $this->rel_categories
                                ON $this->table_name.category_id=$this->rel_categories.id AND $this->rel_categories.deleted=0
                                LEFT JOIN $this->rel_types
                                ON $this->table_name.type_id=$this->rel_types.id AND $this->rel_types.deleted=0
                        	WHERE
				users_last_import.assigned_user_id=
					'{$current_user->id}'
				AND users_last_import.bean_type='ProductTemplates'
				AND users_last_import.bean_id=product_templates.id
				AND users_last_import.deleted=0
                                AND product_templates.deleted=0 ";

		if(! empty($order_by))
		{
			$query .= " ORDER BY $order_by";
		}

		return $query;


	}
}
?>
