<?php
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Professional Subscription
 * Agreement ("License") which can be viewed at
 * http://www.sugarcrm.com/crm/products/sugar-professional-eula.html
 * By installing or using this file, You have unconditionally agreed to the
 * terms and conditions of the License, and You may not use this file except in
 * compliance with the License.  Under the terms of the license, You shall not,
 * among other things: 1) sublicense, resell, rent, lease, redistribute, assign
 * or otherwise transfer Your rights to the Software, and 2) use the Software
 * for timesharing or service bureau purposes such as hosting the Software for
 * commercial gain and/or for the benefit of a third party.  Use of the Software
 * may be subject to applicable fees and any use of the Software without first
 * paying applicable fees is strictly prohibited.  You do not have the right to
 * remove SugarCRM copyrights from the source code or user interface.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *  (i) the "Powered by SugarCRM" logo and
 *  (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * Your Warranty, Limitations of liability and Indemnity are expressly stated
 * in the License.  Please refer to the License for the specific language
 * governing these rights and limitations under the License.  Portions created
 * by SugarCRM are Copyright (C) 2004-2007 SugarCRM, Inc.; All Rights Reserved.
 ********************************************************************************/
//require_once('include/SubPanel/SubPanel.php');
class StudioModule
{
	var $name ;
	var $subpanels = array ( ) ;
	var $views = array ( ) ;
	var $popups = array ( ) ;
	var $providedSubpanels = array ( ) ;
	var $search = array ( ) ;

	var $portal = array ( ) ;

	

	function StudioModule ( $module )
	{
		global $app_list_strings ;
		$moduleNames = array_change_key_case ( $app_list_strings [ 'moduleList' ] ) ;
		
		$this->name = isset($moduleNames [ strtolower ( $module ) ]) ? $moduleNames [ strtolower ( $module ) ] : strtolower ( $module ) ;
		$this->module = $module ;
		$this->getViews () ;
		$this->getSubpanels () ;
		$this->getPopups () ;
		$this->getSearch () ;

		$this->getPortal () ;

	}
	
	/**
	 * Gets a list of views available
	 * ListView/EditView/DetailView
	 *
	 */
	function getViews ()
	{
		$views = array ( 'editviewdefs.php' => array ( 'name' => $GLOBALS [ 'mod_strings' ] [ 'LBL_EDITVIEW' ] , 'type' => 'editView' ) , 'detailviewdefs.php' => array ( 'name' => $GLOBALS [ 'mod_strings' ] [ 'LBL_DETAILVIEW' ] , 'type' => 'detailView' ) , 'listviewdefs.php' => array ( 'name' => $GLOBALS [ 'mod_strings' ] [ 'LBL_LISTVIEW' ] , 'type' => 'list' ) ) ;
		$path = 'modules/' . $this->module . '/metadata' ;
		foreach ( $views as $file => $name )
		{
			if (file_exists ( $path . '/' . $file ))
			{
				$this->views [ $file ] = $name ;
			}
		}
		$this->views [ 'quickcreatedefs.php' ] = array ( 'name' => $GLOBALS [ 'mod_strings' ] [ 'LBL_QUICKCREATE' ] , 'type' => 'quickCreate' ) ;
	}
	function getSearch ()
	{
		$path = 'modules/' . $this->module . '/metadata' ;
		if (file_exists ( $path . '/searchdefs.php' ))
		{
			$this->search [ $GLOBALS [ 'mod_strings' ] [ 'LBL_BASIC_SEARCH' ] ] = 'basic_search' ;
			$this->search [ $GLOBALS [ 'mod_strings' ] [ 'LBL_ADVANCED_SEARCH' ] ] = 'advanced_search' ;
		}
	}
	

	function getPortal ()
	{
		$views = array ( 'editviewdefs.php' => array ( 'name' => $GLOBALS [ 'mod_strings' ] [ 'LBL_EDITVIEW' ] , 'type' => 'editView' ) , 'detailviewdefs.php' => array ( 'name' => $GLOBALS [ 'mod_strings' ] [ 'LBL_DETAILVIEW' ] , 'type' => 'detailView' ) , 'listviewdefs.php' => array ( 'name' => $GLOBALS [ 'mod_strings' ] [ 'LBL_LISTVIEW' ] , 'type' => 'list' ) ) ;
		$path = 'portal/modules/' . $this->module . '/metadata' ;
		foreach ( $views as $file => $name )
		{
			if (file_exists ( $path . '/' . $file ))
			{
				$this->portal [ $file ] = $name ;
			}
		}
	}

	

	/**
	 * Gets a list of popups available
	 *
	 */
	function getPopups ()
	{
		$path = 'modules/' . $this->module . '/metadata' ;
	
	}
	/**
	 * Gets a list of subpanels used by the current module
	 */
	function getSubpanels ()
	{
		global $modListHeader, $beanList, $beanFiles ;
		if (! isset ( $modListHeader ))
		{
			$modListHeader = array ( ) ;
		}
		$this->subpanels = array(); // initialize as some modules do not have subpanels
		require_once ('include/SubPanel/SubPanel.php') ;
		if (file_exists ( "modules/$this->module/metadata/subpaneldefs.php" ))
		{
			$GLOBALS [ 'log' ]->debug ( "StudioModule->getSubpanels(): getting subpanels for " . $this->module ) ;

			$layout_def = SubPanel::getModuleSubpanels ( $this->module ) ;

			foreach ( $layout_def as $sub => $label )
			{
				if ($sub == 'users')
					continue ;

				$name = (! empty ($label)) ? translate ( $label, $this->module ) : $sub ;
				$this->subpanels [ $sub ] = ucfirst ( $name ) ;
			}

		}

	
	}
	/**
	 * gets a list of subpanels provided to other modules
	 *
	 */
	function getProvidedSubpanels ()
	{
		if (file_exists ( 'modules/' . $this->module . '/metadata/subpanels' ))
		{
			$d = dir ( 'modules/' . $this->module . '/metadata/subpanels' ) ;
			while ( $e = $d->read () )
			{
				if (substr ( $e, 0, 1 ) != '.')
				{
					$this->providedSubpanels [ $e ] = $e ;
				}
			}
		}
	}
	
	function getNodes ()
	{
		$nodes = array ( 'name' => $this->name , 'module' => $this->module , 'type' => 'StudioModule' , 'action' => 'module=ModuleBuilder&action=wizard&editModule=' . $this->module , 'children' => array ( array ( 'name' => 'labels' , 'label' => $GLOBALS [ 'mod_strings' ] [ 'LBL_LABELS' ] , 'action' => 'module=ModuleBuilder&action=editLabels&editModule=' . $this->module ) , array ( 'name' => 'fields' , 'label' => $GLOBALS [ 'mod_strings' ] [ 'LBL_FIELDS' ] , 'action' => 'module=ModuleBuilder&action=modulefields&view_package=studio&view_module=' . $this->module ) ) ) ;
		$layouts = array ( ) ;
		foreach ( $this->views as $file => $def )
		{
			$file = str_replace ( $file, '.php', '' ) ;
			$viewType = ($def [ 'type' ] == 'list') ? "ListView" : ucfirst ( $def [ 'type' ] ) ;
			$layouts [] = array ( 'name' => $def [ 'name' ] , 'action' => 'module=ModuleBuilder&action=editLayout&view=' . $viewType . '&editModule=' . $this->module ) ;
		}
		
		$search = array ( ) ;
		foreach ( $this->search as $searchName => $searchType )
		{
			$search [] = array ( 'name' => $searchName , 'action' => 'module=ModuleBuilder&action=editLayout&view=SearchView&editModule=' . $this->module . '&searchlayout=' . $searchType ) ;
		}
		if (! empty ( $search ))
		{
			$layouts [] = array ( 'name' => $GLOBALS [ 'mod_strings' ] [ 'LBL_SEARCH' ] , 'type' => 'Folder' , 'children' => $search , 'action' => 'module=ModuleBuilder&action=wizard&search=1&editModule=' . $this->module ) ;
		}
		
		if (! empty ( $layouts ))
		{
			$nodes [ 'children' ] [] = array ( 'name' => $GLOBALS [ 'mod_strings' ] [ 'LBL_LAYOUTS' ] , 'type' => 'Folder' , 'children' => $layouts , 'action' => 'module=ModuleBuilder&action=wizard&layouts=1&editModule=' . $this->module ) ;
		}
		
		$subpanels = array ( ) ;
		foreach ( $this->subpanels as $sub => $subname )
		{
			$subpanels [] = array ( 'label' => $subname, 'name' => $sub , 'action' => 'module=ModuleBuilder&action=editLayout&view=ListView&editModule=' . $this->module . '&subpanel=' . $sub . '&subpanelLabel=' . $subname ) ;
		}
		if (! empty ( $subpanels ))
		{
			$nodes [ 'children' ] [] = array ( 'name' => $GLOBALS [ 'mod_strings' ] [ 'LBL_SUBPANELS' ] , 'type' => 'Folder' , 'children' => $subpanels , 'action' => 'module=ModuleBuilder&action=wizard&subpanels=1&editModule=' . $this->module ) ;
		}
		

		$portal = array ( ) ;
		foreach ( $this->portal as $file => $def )
		{
			$file = str_replace ( $file, '.php', '' ) ;
			$viewType = ($def [ 'type' ] == 'list') ? "ListView" : ucfirst ( $def [ 'type' ] ) ;
			$portal [] = array ( 'name' => $def [ 'name' ] , 'action' => 'module=ModuleBuilder&action=editPortal&view=' . $viewType . '&editModule=' . $this->module ) ;
		}
		if (! empty ( $portal ))
		{
			$nodes [ 'children' ] [] = array ( 'name' => $GLOBALS [ 'mod_strings' ] [ 'LBL_PORTAL' ] , 'type' => 'Folder' , 'children' => $portal , 'action' => 'module=ModuleBuilder&action=wizard&portal=1&editModule=' . $this->module ) ;
		}

		

		return $nodes ;
	}

}
?>
