<?php
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Professional Subscription
 * Agreement ("License") which can be viewed at
 * http://www.sugarcrm.com/crm/products/sugar-professional-eula.html
 * By installing or using this file, You have unconditionally agreed to the
 * terms and conditions of the License, and You may not use this file except in
 * compliance with the License.  Under the terms of the license, You shall not,
 * among other things: 1) sublicense, resell, rent, lease, redistribute, assign
 * or otherwise transfer Your rights to the Software, and 2) use the Software
 * for timesharing or service bureau purposes such as hosting the Software for
 * commercial gain and/or for the benefit of a third party.  Use of the Software
 * may be subject to applicable fees and any use of the Software without first
 * paying applicable fees is strictly prohibited.  You do not have the right to
 * remove SugarCRM copyrights from the source code or user interface.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *  (i) the "Powered by SugarCRM" logo and
 *  (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * Your Warranty, Limitations of liability and Indemnity are expressly stated
 * in the License.  Please refer to the License for the specific language
 * governing these rights and limitations under the License.  Portions created
 * by SugarCRM are Copyright (C) 2004-2007 SugarCRM, Inc.; All Rights Reserved.
 ********************************************************************************/
require_once ('modules/ModuleBuilder/MB/ModuleBuilder.php');
require_once ('modules/ModuleBuilder/parsers/ParserFactory.php');

class ModuleBuilderController extends SugarController
{
    var $action_remap = array();

    function fromModuleBuilder()
    {
        return (isset($_REQUEST['MB']) && ($_REQUEST['MB'] == '1'));
    }

    function action_editLayout ()
    {
        switch ( strtolower($_REQUEST ['view']))
        {
            case 'editview' :
            case 'detailview' :
            case 'quickcreate' :
                $this->view = 'layoutView';
                break;
            case 'listview' :
                $this->view = 'listView';
                break;
            case 'searchview' :
                $this->view = 'searchView';
                break;
            default :
                $GLOBALS ['log']->fatal('Action = editLayout with unknown view=' . $_REQUEST ['view']);
        }
    }

    function action_ViewTree ()
    {
        require_once ('modules/ModuleBuilder/MB/AjaxCompose.php');
        switch ( $_REQUEST ['tree'])
        {
            case 'ModuleBuilder' :
                require_once ('modules/ModuleBuilder/MB/MBPackageTree.php');
                $mbt = new MBPackageTree();
                break;
            case 'Studio' :
                require_once ('modules/ModuleBuilder/Module/StudioTree.php');
                $mbt = new StudioTree();
        }
        $ajax = new AjaxCompose();
        $ajax->addSection('west', $mbt->getName(), $mbt->fetchNodes());
        echo $ajax->getJavascript();

        sugar_cleanup(true);

    }
    function action_SavePackage ()
    {
        $mb = new ModuleBuilder();
        $load = (! empty($_REQUEST ['original_name'])) ? $_REQUEST ['original_name'] : $_REQUEST ['name'];
        if (! empty($load))
        {
            $mb->getPackage($load);

            if (! empty($_REQUEST ['duplicate']))
            {
                $mb->packages [$load]->copy($_REQUEST ['name']);
            }
            $mb->packages[$load]->populateFromPost();
            $mb->packages[$load]->loadModules();
            $mb->save();
            if (! empty($_REQUEST ['original_name']) && $_REQUEST ['original_name'] != $_REQUEST ['name'])
            {
                if (! $mb->packages[$load]->rename($_REQUEST ['name']))
                {
                    $mb->packages[$load]->name = $_REQUEST ['original_name'];
                    $_REQUEST ['name'] = $_REQUEST ['original_name'];
                }
            }else{
            	$load = $mb->packages[$load]->name;
            }
            $_REQUEST ['package'] = $mb->packages [$load]->name;
            $this->view = 'package';
        }
    }

    function action_BuildPackage ()
    {
        $mb = new ModuleBuilder();
        $load = $_REQUEST ['name'];
        if (! empty($load))
        {
            $mb->getPackage($load);
            $mb->packages [$load]->build();
        }
    }
    function action_DeployPackage(){
        $mb =  new ModuleBuilder();
        $load = $_REQUEST['package'];
        $message = $GLOBALS['mod_strings']['LBL_MODULE_DEPLOYED'];
        if(!empty($load)){
            $zip = $mb->getPackage($load);
            require_once('ModuleInstall/PackageManager/PackageManager.php');
            $pm = new PackageManager();
            $info = $mb->packages[$load]->build(false);
            mkdir_recursive($GLOBALS['sugar_config']['cache_dir'] . '/'. 'upload/upgrades/module/');
            rename($info['zip'], $GLOBALS['sugar_config']['cache_dir'] . '/'. 'upload/upgrades/module/'. $info['name']. '.zip');
            copy($info['manifest'], $GLOBALS['sugar_config']['cache_dir'] . '/'. 'upload/upgrades/module/'. $info['name']. '-manifest.php');
            $_REQUEST['install_file'] = $GLOBALS['sugar_config']['cache_dir'] . '/'. 'upload/upgrades/module/'. $info['name']. '.zip';
            $GLOBALS['mi_remove_tables'] = false;
            $pm->performUninstall($load);
            $pm->performInstall($_REQUEST['install_file']);
        }
		echo 'complete';

    }

    function action_ExportPackage()
    {
        $mb = new ModuleBuilder();
        $load = $_REQUEST ['name'];
        $author = $_REQUEST ['author'];
        $description = $_REQUEST ['description'];
        $readme=$_REQUEST['readme'];
        if (! empty($load))
        {
            $mb->getPackage($load);
            $mb->packages [$load]->author=$author;
            $mb->packages [$load]->description=$description;
            $mb->packages [$load]->exportProject();
            $mb->packages [$load]->readme=$readme;
        }
    }

    function action_DeletePackage ()
    {
        $mb = new ModuleBuilder();
        $mb->getPackage($_REQUEST ['package']);
        $mb->packages [$_REQUEST ['package']]->delete();
        $this->view = 'deletepackage';
    }

    function action_SaveModule ()
    {
        $mb = new ModuleBuilder();
        $load = (! empty($_REQUEST ['original_name'])) ? $_REQUEST ['original_name'] : $_REQUEST ['name'];
        if (! empty($load))
        {
            $mb->getPackage($_REQUEST ['package']);
            $mb->packages [$_REQUEST ['package']]->getModule($load);
            $module = & $mb->packages [$_REQUEST ['package']]->modules [$load];
            $module->populateFromPost();
            $mb->save();
            if (! empty($_REQUEST ['duplicate']))
            {
                $module->copy($_REQUEST ['name']);
            } else if (! empty($_REQUEST ['original_name']) && $_REQUEST ['original_name'] != $_REQUEST ['name'])
            {
                if (! $module->rename($_REQUEST ['name']))
                {
                    $module->name = $_REQUEST ['original_name'];
                    $_REQUEST ['name'] = $_REQUEST ['original_name'];
                }
            }

            $_REQUEST ['view_package'] = $_REQUEST ['package'];
            $_REQUEST ['view_module'] = $module->name;
            $this->view = 'module';
        }
    }
    function action_DeleteModule ()
    {
        $mb = new ModuleBuilder();
        $module = & $mb->getPackageModule($_REQUEST ['package'], $_REQUEST ['view_module']);
        $module->delete();
        $this->view = 'package';
    }

    // TO DO: unifying MB and Studio2 resulted in two saveLabels actions - this needs to be tidied or rethought
    function action_saveLabels ()
    {
        $parser = ParserFactory::getParser('label');
        if (isset($_REQUEST ['view_module'])) //MODULE BUILDER
        {
            $parser->handleSave($_REQUEST, $_REQUEST ['view_module'], $_REQUEST ['selected_lang']);
            $this->view = 'modulelabels';
        } else //STUDIO
        {
            $parser->handleSave($_REQUEST, $_REQUEST ['editModule'], $_REQUEST ['selected_lang']);
            $this->view = isset($_REQUEST ['editLayout']) ? 'edit' : 'labels'; // detect if we are being called by the LayoutEditor rather than the LabelEditor (set in view.layoutlabel.php)
        }
    }

    function action_SaveLabel ()
    {
        if (! empty($_REQUEST ['view_module']))
        {
            $module = $_REQUEST ['view_module'];
            //$label_array = array("label_" . $_REQUEST['label']=>$_REQUEST['label_value']);
            $params = $_REQUEST;
            $params ["label_" . $_REQUEST ['label']] = $_REQUEST ['label_value'];
            $parser = ParserFactory::getParser('label');
            $parser->handleSave($params, $module, 'en_us');
        }
        $this->view = 'modulefields';
    }

    function action_ExportCustom ()
    {
        $modules  = $_REQUEST['modules'];
        $name  = $_REQUEST['name'];
        $author  = $_REQUEST['author'];
        $description  = $_REQUEST['description'];
        ob_clean();
        if(!empty($modules) && !empty($name)){
            require_once('modules/ModuleBuilder/MB/ModuleBuilder.php');
            $mb = new MBPackage($name);
            $mb->author = $author;
            $mb->description = $description;
            $mb->exportCustom($modules,true,true);
        }
    }

    function action_SaveField ()
    {
        require_once ('modules/DynamicFields/FieldCases.php');
        $field = get_widget($_POST ['type']);
        $_POST ['name'] = trim(strtolower($_POST ['name']));
        $field->populateFromPost(); 

        $type = $_REQUEST ['view_package'];
        if ($type == 'studio')
        {
            require_once ('modules/DynamicFields/DynamicField.php');
            if (! empty($_REQUEST ['view_module']))
            {
                $module = $_REQUEST ['view_module'];
                $df = new DynamicField($module);
                $class_name = $GLOBALS ['beanList'] [$module];
                require_once ($GLOBALS ['beanFiles'] [$class_name]);
                $mod = new $class_name();
                $df->setup($mod);

                $field->save($df);
                $this->action_SaveLabel();
            }
        } else
        {
            $vardef = $field->get_field_def();
            unset($vardef ['source']);
            $mb = new ModuleBuilder();
            $module = & $mb->getPackageModule($_REQUEST ['view_package'], $_REQUEST ['view_module']);
            $field->save($module);
            if (isset($_REQUEST ['label']) && isset($_REQUEST ['label_value']))
                $module->setLabel('en_us', $_REQUEST ['label'], $_REQUEST ['label_value']);
//              if (! empty($_REQUEST ['view_module']))
//        {
//            $module = $_REQUEST ['view_module'];
//            //$label_array = array("label_" . $_REQUEST['label']=>$_REQUEST['label_value']);
//            $params = $_REQUEST;
//            $params ["label_" . $_REQUEST ['label']] = $_REQUEST ['label_value'];
//            $parser = ParserFactory::getParser('label');
//            $parser->handleSave($params, $module, 'en_us');
//        }
            $mb->save();
        }
        $this->view = 'modulefields';
    }
    function action_RefreshField ()
    {
        require_once ('modules/DynamicFields/FieldCases.php');
        $field = get_widget($_POST ['type']);
        $field->populateFromPost();
        $this->view = 'modulefield';
    }

    function action_SaveRelationship ()
    {
        $mb = new ModuleBuilder();
        $module = & $mb->getPackageModule($_REQUEST ['view_package'], $_REQUEST ['view_module']);
        $rel = $module->mbrelationship->relationshipFromPost();
        $module->mbrelationship->add($rel);
        $module->setLabel('en_us', 'lbl_' . $rel ['name'], $rel ['label']);
        $mb->save();
        $this->view = 'relationships';
    }
    function action_DeleteRelationship ()
    {
        $mb = new ModuleBuilder();
        $module = & $mb->getPackageModule($_REQUEST ['view_package'], $_REQUEST ['view_module']);
        $rel = $module->mbrelationship->relationshipFromPost();
        $module->mbrelationship->delete($rel ['name']);
        $mb->save();
        $this->view = 'relationships';
    }

    function action_SaveDropDown ()
    {
        $parser = ParserFactory::getParser('dropdown');
        $parser->saveDropDown($_REQUEST);
        $this->view = 'dropdowns';
    }

    function action_DeleteField ()
    {
        $type = $_REQUEST ['view_package'];
        $field = get_widget($_POST['type']);
        $field->name = $_REQUEST['name'];
        if ($type == 'studio')
        {
            if (! empty($_REQUEST ['name']) && ! empty($_REQUEST ['view_module']))
            {
                require_once ('modules/DynamicFields/DynamicField.php');
                $module = $_REQUEST ['view_module'];
                $class_name = $GLOBALS ['beanList'] [$module];
                require_once ($GLOBALS ['beanFiles'] [$class_name]);
                $seed = new $class_name();
//                $temp = $GLOBALS['external_cache_object']->get("VardefManager".$module.$class_name);
//                $GLOBALS['log']->debug("cache for ".$module.$class_name." initially = ".$temp);
                $df = new DynamicField($module);
                $df->setup($seed);
                $field->delete($df);
//                $temp = $GLOBALS['external_cache_object']->get("VardefManager".$module.$class_name);
//                $GLOBALS['log']->debug("cache now = ".$temp);
            }
        } else
        {
            $mb = new ModuleBuilder();
            $module = & $mb->getPackageModule($_REQUEST ['view_package'], $_REQUEST ['view_module']);
            $field->delete($module);
            $mb->save();
        }
        $this->view = 'modulefields';
    }

    function action_CloneField ()
    {
        $this->view_object_map ['field_name'] = $_REQUEST ['name'];
        $this->view_object_map ['is_clone'] = true;
        $this->view = 'modulefield';
    }

    function action_SaveAssistantPref ()
    {
		global $current_user;
		if (isset($_REQUEST ['pref_value'])){
			if($_REQUEST['pref_value'] == 'ignore'){
				$current_user->setPreference('mb_assist', 'DISABLED', 0, 'Assistant') ;
			} else{
				$current_user->setPreference('mb_assist', 'ENABLED', 0, 'Assistant');
			}
			$current_pref=$current_user->getPreference('mb_assist', 'Assistant');
			echo "Assistant.processUserPref('$current_pref')";
			sugar_cleanup(true); //push preferences to DB.
                }
            }
    // Studio2 Actions


    function action_EditProperty ()
    {
        $this->view = 'property';
    }

    function action_saveProperty()
    {
        $parser = ParserFactory::getParser('label');
        $parser->init($_REQUEST['editModule']);
        $parser->handleSave($_REQUEST, ($_REQUEST['subpanel'] != "") ? $_REQUEST['subpanel'] : $_REQUEST['editModule'], $_REQUEST['selected_lang']);
        $this->view = 'closeeast';
    }

    function action_editModule ()
    {
        $this->view = 'module';
    }

    function action_wizard ()
    {
        $this->view = 'wizard';
    }

    /**
     * Receive a layout through $_REQUEST and save it out to the working files directory
     * Expects a series of $_REQUEST parameters all in the format $_REQUEST['slot-panel#-slot#-property']=value
     */

    function action_saveLayout ()
    {

        if(isset($_REQUEST['PORTAL']))
        {
            $parser = ParserFactory::getParser('portallayoutview');
            $this->view = 'portallayoutview';
        }
        else
        {

            $parser = ParserFactory::getParser('layoutview', $this->fromModuleBuilder());
            $this->view = 'layoutview';

        }

        $parser->init($_REQUEST ['editModule'], $_REQUEST ['view'], true);
        $parser->writeWorkingFile();
    }

    function action_saveAndPublishLayout ()
    {

        if(isset($_REQUEST['PORTAL']))
        {
            $parser = ParserFactory::getParser('portallayoutview');
            $this->view = 'portallayoutview';
        }
        else
        {

            $parser = ParserFactory::getParser('layoutview', $this->fromModuleBuilder());
            $this->view = 'layoutview';

        }

        $parser->init($_REQUEST ['editModule'], $_REQUEST ['view'], true);
        $parser->writeWorkingFile();
        $parser->handleSave();
    }

    function action_manageBackups ()
    {

    }


    function action_listViewSave ()
    {

        if(isset($_REQUEST['PORTAL']))
        {
            $this->view = 'portallistview';
            $lv = ParserFactory::getParser('portallistview');
            $lv->init($_REQUEST ['editModule']);
        }
        else
        {

            $this->view = 'listview';
            if (! empty($_REQUEST ['subpanel']))
            {
                $parserName = (! empty($_REQUEST ['local'])) ? 'localsubpanel' : 'subpanel';
                $lv = ParserFactory::getParser($parserName, ! empty($_REQUEST ['MB']));
                $lv->init($_REQUEST ['editModule'], $_REQUEST ['subpanel']);
                $GLOBALS['log']->debug("in action_listviewsave - parser = $parserName");
            }
            else
            {
                $lv = ParserFactory::getParser('listview', $this->fromModuleBuilder());
                $lv->init($_REQUEST ['editModule']);
                $GLOBALS['log']->debug("in action_listviewsave - parser = listview");
            }

        }

        $lv->handleSave();
    }

    function action_searchViewSave ()
    {
        $lv = ParserFactory::getParser('searchview', $this->fromModuleBuilder());
        $lv->init($_REQUEST ['editModule'],$_REQUEST['searchlayout']);
        $lv->handleSave();
        $this->view = 'searchView';
    }

    function action_editLabels ()
    {
        $this->view = 'labels';
    }
	
    function action_get_app_list_string ()
    {
    	require_once('include/JSON.php');
        $json = new JSON();
        if (isset($_REQUEST['key']) && !empty($_REQUEST['key'])) {
        	$key = $_REQUEST['key'];
        	$value = array();
        	if (!empty($GLOBALS['app_list_strings'][$key])) {
        		$value = $GLOBALS['app_list_strings'][$key];
        	} else {
	        	$package_strings = array();
				if(!empty($_REQUEST['view_package']) && $_REQUEST['view_package'] != 'studio' && !empty($_REQUEST['view_module'])){
					require_once('modules/ModuleBuilder/MB/ModuleBuilder.php');
					$mb = new ModuleBuilder();
					$module =& $mb->getPackageModule($_REQUEST['view_package'], $_REQUEST['view_module']);
					$lang = $GLOBALS['current_language'];
					$module->mblanguage->generateAppStrings(false);
					$package_strings = $module->mblanguage->appListStrings[$lang.'.lang.php'];
					if (isset($package_strings[$key]) && is_array($package_strings[$key])) {
						$value = $package_strings[$key];
					}
				}
        	}
        	echo $json->encode($value);
        }
    }


    function action_editPortal()
    {
        switch ( strtolower($_REQUEST ['view']))
        {
            case 'editview' :
            case 'detailview' :
                $this->view = 'portallayoutView';
                break;
            case 'listview' :
                $this->view = 'portallistView';
                break;
            case 'searchview' :
                $this->view = 'portalsearchView';
                break;
            default :
                $GLOBALS ['log']->fatal('Action = editPortal with unknown view=' . $_REQUEST ['view']);
        }
    }
    function action_portalstylesave()
    {
		if(strcmp(strtolower($_FILES['filename']['type']),'text/css') == 0)
		{
			mkdir_recursive('custom/portal/custom', true);
			move_uploaded_file($_FILES['filename']['tmp_name'], 'custom/portal/custom/style.css');
			$_REQUEST['label'] = 'LBL_SP_UPLOADED';
			$GLOBALS['log']->debug('action_portalstylesave(): saving file '.$_FILES['filename']['name']);
		}
		else
		{
			$_REQUEST['label'] = 'ERROR_SP_UPLOADED';
			$GLOBALS['log']->error('action_portalstylesave(): error uploading '.$_FILES['filename']['tmp_name']);
		}

		@unlink($_FILES['filename']['tmp_name']);
        $this->view = 'portalstyleupload';
    }

    function action_portalsyncsync()
    {
        $option = str_replace(array('soap.php', 'index.php', 'portal_sync.php'), '', $_REQUEST['portalURL'] );
		$GLOBALS['system_config']->saveSetting('system', 'portal_url', $option);
		$GLOBALS['system_config']->settings['system_portal_url'] = $option;
		$this->view = 'portalsync';
    }


}
?>
