/**
 * The contents of this file are subject to the SugarCRM Professional Subscription
 * Agreement ("License") which can be viewed at
 * http://www.sugarcrm.com/crm/products/sugar-professional-eula.html
 * By installing or using this file, You have unconditionally agreed to the
 * terms and conditions of the License, and You may not use this file except in
 * compliance with the License.  Under the terms of the license, You shall not,
 * among other things: 1) sublicense, resell, rent, lease, redistribute, assign
 * or otherwise transfer Your rights to the Software, and 2) use the Software
 * for timesharing or service bureau purposes such as hosting the Software for
 * commercial gain and/or for the benefit of a third party.  Use of the Software
 * may be subject to applicable fees and any use of the Software without first
 * paying applicable fees is strictly prohibited.  You do not have the right to
 * remove SugarCRM copyrights from the source code or user interface.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *  (i) the "Powered by SugarCRM" logo and
 *  (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * Your Warranty, Limitations of liability and Indemnity are expressly stated
 * in the License.  Please refer to the License for the specific language
 * governing these rights and limitations under the License.  Portions created
 * by SugarCRM are Copyright (C) 2004-2007 SugarCRM, Inc.; All Rights Reserved.
 */


Studio2.PanelDD = function(id, sGroup) {
	Studio2.PanelDD.superclass.constructor.call(this, id, sGroup);

    var el = this.getDragEl();
	Dom.setStyle(el, 'opacity', 0.67); // The proxy is slightly transparent
    this.goingUp = false;
    this.lastY = 0;
};

	
Ext.extend(Studio2.PanelDD, Ext.dd.DDProxy, {

    startDrag: function(x, y) { 	
        // make the proxy look like the source element
		var dragEl = this.getDragEl();
		var clickEl = this.getEl();
//		dragEl.innerHTML = clickEl.innerHTML; // this includes id's which causes problems for YUI - it often doesn't cleanly remove the dragEl at the end of the drag
		dragEl.className = clickEl.className;
		dragEl.innerHTML = '';
		var title = document.createElement('h3');
		title.appendChild(document.createTextNode(document.getElementById("le_panellabel_"+clickEl.id).childNodes[0].nodeValue));
		dragEl.appendChild(title);
		this.deletePanel = false;
		Studio2.copyId = null;

		if (Studio2.establishLocation(clickEl) == 'toolbox') {
			var copy = Studio2.newPanel();
			Studio2.setCopy(copy);
			clickEl.parentNode.insertBefore(copy,clickEl.nextSibling);

			Dom.setStyle(copy,'display','block'); // must make it visible - the css sets rows outside of panel to invisible
			Dom.setStyle(clickEl,"display","none");
		}

		Dom.setStyle(clickEl, "visibility", "hidden");

    },

    endDrag: function(e) {
		ModuleBuilder.state.isDirty=true;
//  	alert("endDrag");
     
        var srcEl = this.getEl();
        var proxy = this.getDragEl();      
        var proxyid = proxy.id;
        var thisid = this.id;
        
        if (this.deletePanel) {
			Studio2.removeElement(srcEl);
			// If we've just removed the last panel then we need to put an empty panel back in
			proxy.innerHTML = '';
        	Studio2.tidyPanels();
        } else {
        
	        // Show the proxy element and animate it to the src element's location
	        Dom.setStyle(proxy, "visibility", "");
	        Dom.setStyle(srcEl, "display",""); // display!=none for getXY to work
	        
			Ext.get(proxy).alignTo(srcEl, 'tl', null, {callback:function(){
				Dom.setStyle(proxyid, "visibility", "hidden");
	            Dom.setStyle(thisid, "visibility", "");
			}});
	
			if (Studio2.isSpecial(srcEl)) {
				if (Studio2.establishLocation(srcEl) == 'panels') {
					// dropping on the panels means that the panel is no longer special
					Studio2.unsetSpecial(srcEl);
					// add in the template row to the new panel
					var newRow = Studio2.newRow(false);
					srcEl.appendChild(newRow);
					// bug 16470: change the panel title to make it unique
					var title = document.getElementById("le_panellabel_"+srcEl.id);
					title.childNodes[0].nodeValue =  'PANEL '+ ++Studio2.panelNumber;
					Studio2.activateElement(newRow);
					Studio2.setSpecial(Studio2.copy());
					Studio2.activateCopy();
					Dom.setStyle(Studio2.copy(),'display','block');
				}
				else
				{
					// we have a special panel that hasn't been moved to the panels area - invalid drop, so remove the copy if there is one
					var copy = document.getElementById(Studio2.copyId);
					copy.parentNode.removeChild(copy);
					Studio2.copyID = null;
				}

			}
		}

    },

	onInvalidDrop: function(e) {
//		alert("invalid");
		var srcEl = this.getEl();
		var dragEl = this.getDragEl();
		dragEl.innerHTML = '';
	},
	
    onDragDrop: function(e, id) {
//		alert("ondragdrop");
		
		var srcEl = this.getEl();
		var destEl = Ext.get(id).dom; // where this element is being dropped
		
		
		// if source was in a panel (not toolbox) and destination is the delete area then remove this element
		if ((Studio2.establishLocation(srcEl) == 'panels') && (Studio2.establishLocation(destEl) == 'delete')) {
			this.deletePanel = true;
			//Studio2.removeElement(srcEl);
		}
    },

    onDrag: function(e) {
        // Keep track of the direction of the drag for use during onDragOver
        var y = e.getPageY();

        if (y < this.lastY) {
            this.goingUp = true;
        } else if (y > this.lastY) {
            this.goingUp = false;
        }

        this.lastY = y;
    },

    onDragOver: function(e, id) {
        var srcEl = this.getEl();
//		var destEl = YAHOO.util.Dom.get(YAHOO.util.DDM.getBestMatch(dds).id);
        var destEl = Dom.get(id);

        if ((Studio2.establishLocation(destEl) == 'panels') && (destEl.className.indexOf('le_panel') != -1)) {
        	Dom.setStyle(srcEl,'visibility','hidden');
        	Dom.setStyle(srcEl,'display','block');
        	var orig_p = srcEl.parentNode;
            var p = destEl.parentNode;

            if (this.goingUp) {
				p.insertBefore(srcEl, destEl); // insert above
            } else {
                p.insertBefore(srcEl, destEl.nextSibling); // insert below
            }
            Ext.dd.DragDropMgr.refreshCache({'studio2.panels':true});
			//YAHOO.util.DragDropMgr.refreshCache();
        }

    }
});


