<?php
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Professional Subscription
 * Agreement ("License") which can be viewed at
 * http://www.sugarcrm.com/crm/products/sugar-professional-eula.html
 * By installing or using this file, You have unconditionally agreed to the
 * terms and conditions of the License, and You may not use this file except in
 * compliance with the License.  Under the terms of the license, You shall not,
 * among other things: 1) sublicense, resell, rent, lease, redistribute, assign
 * or otherwise transfer Your rights to the Software, and 2) use the Software
 * for timesharing or service bureau purposes such as hosting the Software for
 * commercial gain and/or for the benefit of a third party.  Use of the Software
 * may be subject to applicable fees and any use of the Software without first
 * paying applicable fees is strictly prohibited.  You do not have the right to
 * remove SugarCRM copyrights from the source code or user interface.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *  (i) the "Powered by SugarCRM" logo and
 *  (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * Your Warranty, Limitations of liability and Indemnity are expressly stated
 * in the License.  Please refer to the License for the specific language
 * governing these rights and limitations under the License.  Portions created
 * by SugarCRM are Copyright (C) 2004-2007 SugarCRM, Inc.; All Rights Reserved.
 ********************************************************************************/

require_once ('modules/ModuleBuilder/parsers/parser.modifylistview.php');

class ParserBuildLocalSubpanel extends ParserModifyListView
{
    
    var $columns = array ( 'LBL_DEFAULT' => 'getDefaultFields' , 'LBL_AVAILABLE' => 'getAvailableFields' ) ;
	
	function init ($module_name, $subpanelName)
	{
		$this->package = isset($_REQUEST ['package']) ? $_REQUEST ['package'] : '';
		$this->parent_module = $module_name;
		$this->subpanelName = $subpanelName;
		
		//get the bean from ModuleBuilder
		$mb = new ModuleBuilder();
		$this->module = & $mb->getPackageModule($this->package, $module_name);
		$this->module->mbvardefs->updateVardefs();
		$this->module->field_defs = & $this->module->mbvardefs->vardefs ['fields'];
		
		$subpanel_layout = $this->module->getAvailibleSubpanelDef($this->subpanelName);
		$GLOBALS['log']->debug($subpanel_layout);
		$this->listViewDefs = & $subpanel_layout ['list_fields'];
		$this->originalListViewDefs = $this->listViewDefs;
		$this->language_module = '';
		global $mod_strings;
		$mod_strings = array_merge($mod_strings, $this->module->getModStrings());
	}
	
	function getDefaultFields ()
	{
	    return $this->listViewDefs;
	}
	
    function getAvailableFields ()
    {
        $available = array();
        $lowerFieldList = array_change_key_case ( $this->listViewDefs ) ;
        $GLOBALS['log']->debug($lowerFieldList);
        foreach ( $this->module->field_defs as $key => $def )
        {
            $key = strtolower ( $key ) ;
            if (! isset ( $lowerFieldList [ $key ] ))
            {
                $available [ $key ] = $def ;
            }
        }
		return $available ;
    }

	function handleSave ()
	{

        $newFields = array();
        foreach ($this->listViewDefs as $name => $field)
        {
            if (isset($field['usage']) && $field['usage'] == 'query_only')
            {
                $newFields[$name] = $field;
            }
        }
        
        foreach ($_REQUEST['group_0'] as $field)
        {
            if (! empty($this->originalListViewDefs[$field]))
            {
                $newFields[$field] = $this->originalListViewDefs[$field];
            } 
            else
            {
                $newFields[$field] = array('name' => $field, 'vname' => '');
            }
            
            if (isset($_REQUEST[strtolower($field) . 'width']))
            {
                $width = substr($_REQUEST[strtolower($field) . 'width'], 6, 3);
                if (strpos($width, "%") != false)
                {
                    $width = substr($width, 0, 2);
                }
                if ($width < 101 && $width > 0)
                {
                    $newFields[$field]['width'] = $width;
                }
            }
            else if (isset($this->listViewDefs[$field]['width']))
            {
                $newFields[$field]['width'] = $this->listViewDefs[$field]['width'];
            }
        }
	    $layout = $this->module->getAvailibleSubpanelDef($this->subpanelName);
        $layout['list_fields'] = $newFields;
		$GLOBALS['log']->debug("ParserBuildLocalSubpanel: about to save this layout:");
		$GLOBALS['log']->debug($layout);
		$this->module->saveAvailibleSubpanelDef($this->subpanelName,$layout);
	}
}

?>
