<?php
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Professional Subscription
 * Agreement ("License") which can be viewed at
 * http://www.sugarcrm.com/crm/products/sugar-professional-eula.html
 * By installing or using this file, You have unconditionally agreed to the
 * terms and conditions of the License, and You may not use this file except in
 * compliance with the License.  Under the terms of the license, You shall not,
 * among other things: 1) sublicense, resell, rent, lease, redistribute, assign
 * or otherwise transfer Your rights to the Software, and 2) use the Software
 * for timesharing or service bureau purposes such as hosting the Software for
 * commercial gain and/or for the benefit of a third party.  Use of the Software
 * may be subject to applicable fees and any use of the Software without first
 * paying applicable fees is strictly prohibited.  You do not have the right to
 * remove SugarCRM copyrights from the source code or user interface.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *  (i) the "Powered by SugarCRM" logo and
 *  (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * Your Warranty, Limitations of liability and Indemnity are expressly stated
 * in the License.  Please refer to the License for the specific language
 * governing these rights and limitations under the License.  Portions created
 * by SugarCRM are Copyright (C) 2004-2007 SugarCRM, Inc.; All Rights Reserved.
 ********************************************************************************/

require_once('modules/ModuleBuilder/parsers/parser.modifysubpanel.php');

class ParserBuildSubPanel extends ParserModifySubPanel {
    
	var $listViewDefs = false;
	var $defaults = array();
	var $additional = array();
	var $available = array();
	var $columns = array('LBL_DEFAULT'=>'getDefaultFields', 'LBL_AVAILABLE'=>'getAvailableFields');

	function init($module_name, $child_module){
		global $focus;
		$package_name =$_REQUEST['package'];
		$this->package = $package_name;
		$this->parent_module = $module_name;
		$this->child_module = $child_module;
		global $beanList, $beanFiles;

		//get the bean from ModuleBuilder
		$mb = new ModuleBuilder();
		$mod =& $mb->getPackageModule($package_name, $module_name);
		$mod->mbvardefs->updateVardefs();

		$this->focus = $mod;

		//load the relationships
		$relations = $mod->mbrelationship->relationships;

		foreach ($relations as $relName=>$relData) {
			if ($relData['relate'] == $this->child_module && file_exists("modules/{$relData['relate']}/metadata/subpanels/{$relData['msub']}.php")){
				$this->rbsub = $relData['rsub'];
				$customFile = "custom/modules/{$relData['relate']}/metadata/subpanels/{$mod->name}{$relData['rsub']}.php";
				if (file_exists($customFile)) {
					include($customFile);
				} else {
					include("modules/{$relData['relate']}/metadata/subpanels/{$relData['rsub']}.php");
				}
				$this->list_fields = $subpanel_layout['list_fields'];
			}
		}
		$this->original_list_fields = $this->list_fields;

		$class = $beanList[$child_module];
		require_once($beanFiles[$class]);
		$subBean = new $class();
		$this->panelFieldDefs = $subBean->field_name_map;

		$this->language_module = $child_module;
	}

	function getAvailableFields(){
		$this->availableFields = array();
		foreach($this->original_list_fields as $key=>$def){
			if(!isset($this->list_fields[$key])){
				if(!empty($def['vname']))$def['label'] = $def['vname'];
				$this->availableFields[$key] = $def;
			}
		}

		foreach($this->panelFieldDefs as $key=>$def){
			if((empty($def['source']) || $def['source'] == 'db' || !empty($def['custom_type'])) && empty($this->list_fields[$key])){
				$this->availableFields[$key] = array('width' => '10', 'label'=> $def['vname'] );
			}
		}
		return $this->availableFields;
	}


	function handleSave(){
		require_once('include/SubPanel/SubPanel.php');

		//_ppd($this->original_list_fields);
		//_ppd($this->panelFieldDefs);
		$newFields = array();
		$existingFields = array();
		foreach($this->list_fields as $name => $field){
			if(!isset($field['usage'])|| $field['usage'] != 'query_only'){
				$existingFields[$name] = $field;

			}else{
				$newFields[$name] = $field;
			}
		}

		global $beanList;
		$cMod = !preg_match("/^[a-z]/", $this->child_module) ? $this->child_module : strtoupper($this->child_module[0]) . substr($this->child_module, 1);
		$class = $beanList[$cMod];
		$mod = new $class();
			
		foreach($_REQUEST['group_0'] as $field){
			if(!empty($this->original_list_fields[$field])){
				$newFields[$field] = $this->original_list_fields[$field];
			}else{
				$vname = '';
				if(isset($this->panelFieldDefs[$field])){
					$vname = $this->panelFieldDefs[$field]['vname'];
				}

				if(isset($mod->field_name_map[$field]) &&
				($mod->field_name_map[$field]['type'] == 'bool' || (isset($mod->field_name_map[$field]['custom_type']) && $mod->field_name_map[$field]['custom_type'] == 'bool'))) {
					$newFields[$field] = array('name'=>$field, 'vname'=>$vname, 'widget_type'=>'checkbox');
				} else {
					$newFields[$field] = array('name'=>$field, 'vname'=>$vname);
				}
			}

			if (isset($_REQUEST[strtolower($field).'width'])) {
				$width = substr($_REQUEST[strtolower($field).'width'], 6, 3);
				if (strpos($width, "%") != false) {
					$width = substr($width, 0, 2);
				}
				if ($width < 101 && $width > 0) {
					$newFields[$field]['width'] = $width;
				}
			} else if (isset($this->panelFieldDefs[$field]['width'])) {
				$newFields[$field]['width'] = $this->panelFieldDefs[$field]['width'];
			} else if (!isset($newFields[$field]['width'])){
				$newFields[$field]['width'] = '10';
			}
		}
		$dirname = "custom/modules/{$this->child_module}/metadata/subpanels/";
		if (!is_dir($dirname)) {
			mkdir_recursive($dirname, true);
		}
		$filename = $dirname.$this->focus->name.$this->rbsub.".php";
		if(!write_array_to_file("subpanel_layout['list_fields']", $newFields, $filename)) {
			$GLOBALS['log']->fatal("Could not write $filename");
		}
	}
}

?>
